
/* Generated data (by glib-mkenums) */

#ifndef H_GWH_ENUM_TYPES
#define H_GWH_ENUM_TYPES

#include <glib-object.h>

/* include all headers that may introduce new enums */
#include "gwh-browser.h"
#include "gwh-settings.h"
#include "gwh-utils.h"

G_BEGIN_DECLS

/* enumerations from "gwh-browser.h" */
G_GNUC_INTERNAL
GType gwh_browser_position_get_type (void) G_GNUC_CONST;
#define GWH_TYPE_BROWSER_POSITION (gwh_browser_position_get_type ())

/* enumerations from "gwh-settings.h" */
G_GNUC_INTERNAL
GType gwh_settings_notify_flags_get_type (void) G_GNUC_CONST;
#define GWH_TYPE_SETTINGS_NOTIFY_FLAGS (gwh_settings_notify_flags_get_type ())

/* enumerations from "gwh-utils.h" */
G_GNUC_INTERNAL
GType gwh_window_type_get_type (void) G_GNUC_CONST;
#define GWH_TYPE_WINDOW_TYPE (gwh_window_type_get_type ())
G_END_DECLS

#endif /* guard */

/* Generated data ends here */

