
/*
 *******************  !!! IMPORTANT !!!  ***************************
 *
 * This is a machine generated file, do not edit by hand!
 * If you need to modify this file, see "geanylua/util/mkiface.lua"
 *
 *******************************************************************
 *
*/


typedef enum {
	SLT_VOID, 
	SLT_INT, 
	SLT_STRING, 
	SLT_CELLS, 
	SLT_BOOL, 
	SLT_TEXTRANGE, 
	SLT_STRINGRESULT, 
	SLT_FINDTEXT, 
	SLT_FORMATRANGE, 
	SLT_LAST
} GlspiType;


typedef struct _SciCmdHashEntry {
	const gchar *name;
	GlspiType result;
	gint msgid;
	GlspiType wparam;
	GlspiType lparam;
} SciCmdHashEntry;


static SciCmdHashEntry sci_cmd_hash_entries[] = {
	{"ADDTEXT", SLT_VOID, SCI_ADDTEXT, SLT_INT, SLT_STRING},
	{"ADDSTYLEDTEXT", SLT_VOID, SCI_ADDSTYLEDTEXT, SLT_INT, SLT_CELLS},
	{"INSERTTEXT", SLT_VOID, SCI_INSERTTEXT, SLT_INT, SLT_STRING},
	{"CHANGEINSERTION", SLT_VOID, SCI_CHANGEINSERTION, SLT_INT, SLT_STRING},
	{"CLEARALL", SLT_VOID, SCI_CLEARALL, SLT_VOID, SLT_VOID},
	{"DELETERANGE", SLT_VOID, SCI_DELETERANGE, SLT_INT, SLT_INT},
	{"CLEARDOCUMENTSTYLE", SLT_VOID, SCI_CLEARDOCUMENTSTYLE, SLT_VOID, SLT_VOID},
	{"GETLENGTH", SLT_INT, SCI_GETLENGTH, SLT_VOID, SLT_VOID},
	{"GETCHARAT", SLT_INT, SCI_GETCHARAT, SLT_INT, SLT_VOID},
	{"GETCURRENTPOS", SLT_INT, SCI_GETCURRENTPOS, SLT_VOID, SLT_VOID},
	{"GETANCHOR", SLT_INT, SCI_GETANCHOR, SLT_VOID, SLT_VOID},
	{"GETSTYLEAT", SLT_INT, SCI_GETSTYLEAT, SLT_INT, SLT_VOID},
	{"REDO", SLT_VOID, SCI_REDO, SLT_VOID, SLT_VOID},
	{"SETUNDOCOLLECTION", SLT_VOID, SCI_SETUNDOCOLLECTION, SLT_BOOL, SLT_VOID},
	{"SELECTALL", SLT_VOID, SCI_SELECTALL, SLT_VOID, SLT_VOID},
	{"SETSAVEPOINT", SLT_VOID, SCI_SETSAVEPOINT, SLT_VOID, SLT_VOID},
	{"GETSTYLEDTEXT", SLT_INT, SCI_GETSTYLEDTEXT, SLT_VOID, SLT_TEXTRANGE},
	{"CANREDO", SLT_BOOL, SCI_CANREDO, SLT_VOID, SLT_VOID},
	{"MARKERLINEFROMHANDLE", SLT_INT, SCI_MARKERLINEFROMHANDLE, SLT_INT, SLT_VOID},
	{"MARKERDELETEHANDLE", SLT_VOID, SCI_MARKERDELETEHANDLE, SLT_INT, SLT_VOID},
	{"GETUNDOCOLLECTION", SLT_BOOL, SCI_GETUNDOCOLLECTION, SLT_VOID, SLT_VOID},
	{"GETVIEWWS", SLT_INT, SCI_GETVIEWWS, SLT_VOID, SLT_VOID},
	{"SETVIEWWS", SLT_VOID, SCI_SETVIEWWS, SLT_INT, SLT_VOID},
	{"GETTABDRAWMODE", SLT_INT, SCI_GETTABDRAWMODE, SLT_VOID, SLT_VOID},
	{"SETTABDRAWMODE", SLT_VOID, SCI_SETTABDRAWMODE, SLT_INT, SLT_VOID},
	{"POSITIONFROMPOINT", SLT_INT, SCI_POSITIONFROMPOINT, SLT_INT, SLT_INT},
	{"POSITIONFROMPOINTCLOSE", SLT_INT, SCI_POSITIONFROMPOINTCLOSE, SLT_INT, SLT_INT},
	{"GOTOLINE", SLT_VOID, SCI_GOTOLINE, SLT_INT, SLT_VOID},
	{"GOTOPOS", SLT_VOID, SCI_GOTOPOS, SLT_INT, SLT_VOID},
	{"SETANCHOR", SLT_VOID, SCI_SETANCHOR, SLT_INT, SLT_VOID},
	{"GETCURLINE", SLT_INT, SCI_GETCURLINE, SLT_INT, SLT_STRINGRESULT},
	{"GETENDSTYLED", SLT_INT, SCI_GETENDSTYLED, SLT_VOID, SLT_VOID},
	{"CONVERTEOLS", SLT_VOID, SCI_CONVERTEOLS, SLT_INT, SLT_VOID},
	{"GETEOLMODE", SLT_INT, SCI_GETEOLMODE, SLT_VOID, SLT_VOID},
	{"SETEOLMODE", SLT_VOID, SCI_SETEOLMODE, SLT_INT, SLT_VOID},
	{"STARTSTYLING", SLT_VOID, SCI_STARTSTYLING, SLT_INT, SLT_INT},
	{"SETSTYLING", SLT_VOID, SCI_SETSTYLING, SLT_INT, SLT_INT},
	{"GETBUFFEREDDRAW", SLT_BOOL, SCI_GETBUFFEREDDRAW, SLT_VOID, SLT_VOID},
	{"SETBUFFEREDDRAW", SLT_VOID, SCI_SETBUFFEREDDRAW, SLT_BOOL, SLT_VOID},
	{"SETTABWIDTH", SLT_VOID, SCI_SETTABWIDTH, SLT_INT, SLT_VOID},
	{"GETTABWIDTH", SLT_INT, SCI_GETTABWIDTH, SLT_VOID, SLT_VOID},
	{"CLEARTABSTOPS", SLT_VOID, SCI_CLEARTABSTOPS, SLT_INT, SLT_VOID},
	{"ADDTABSTOP", SLT_VOID, SCI_ADDTABSTOP, SLT_INT, SLT_INT},
	{"GETNEXTTABSTOP", SLT_INT, SCI_GETNEXTTABSTOP, SLT_INT, SLT_INT},
	{"SETCODEPAGE", SLT_VOID, SCI_SETCODEPAGE, SLT_INT, SLT_VOID},
	{"GETIMEINTERACTION", SLT_INT, SCI_GETIMEINTERACTION, SLT_VOID, SLT_VOID},
	{"SETIMEINTERACTION", SLT_VOID, SCI_SETIMEINTERACTION, SLT_INT, SLT_VOID},
	{"MARKERDEFINE", SLT_VOID, SCI_MARKERDEFINE, SLT_INT, SLT_INT},
	{"MARKERSETFORE", SLT_VOID, SCI_MARKERSETFORE, SLT_INT, SLT_INT},
	{"MARKERSETBACK", SLT_VOID, SCI_MARKERSETBACK, SLT_INT, SLT_INT},
	{"MARKERSETBACKSELECTED", SLT_VOID, SCI_MARKERSETBACKSELECTED, SLT_INT, SLT_INT},
	{"MARKERENABLEHIGHLIGHT", SLT_VOID, SCI_MARKERENABLEHIGHLIGHT, SLT_BOOL, SLT_VOID},
	{"MARKERADD", SLT_INT, SCI_MARKERADD, SLT_INT, SLT_INT},
	{"MARKERDELETE", SLT_VOID, SCI_MARKERDELETE, SLT_INT, SLT_INT},
	{"MARKERDELETEALL", SLT_VOID, SCI_MARKERDELETEALL, SLT_INT, SLT_VOID},
	{"MARKERGET", SLT_INT, SCI_MARKERGET, SLT_INT, SLT_VOID},
	{"MARKERNEXT", SLT_INT, SCI_MARKERNEXT, SLT_INT, SLT_INT},
	{"MARKERPREVIOUS", SLT_INT, SCI_MARKERPREVIOUS, SLT_INT, SLT_INT},
	{"MARKERDEFINEPIXMAP", SLT_VOID, SCI_MARKERDEFINEPIXMAP, SLT_INT, SLT_STRING},
	{"MARKERADDSET", SLT_VOID, SCI_MARKERADDSET, SLT_INT, SLT_INT},
	{"MARKERSETALPHA", SLT_VOID, SCI_MARKERSETALPHA, SLT_INT, SLT_INT},
	{"SETMARGINTYPEN", SLT_VOID, SCI_SETMARGINTYPEN, SLT_INT, SLT_INT},
	{"GETMARGINTYPEN", SLT_INT, SCI_GETMARGINTYPEN, SLT_INT, SLT_VOID},
	{"SETMARGINWIDTHN", SLT_VOID, SCI_SETMARGINWIDTHN, SLT_INT, SLT_INT},
	{"GETMARGINWIDTHN", SLT_INT, SCI_GETMARGINWIDTHN, SLT_INT, SLT_VOID},
	{"SETMARGINMASKN", SLT_VOID, SCI_SETMARGINMASKN, SLT_INT, SLT_INT},
	{"GETMARGINMASKN", SLT_INT, SCI_GETMARGINMASKN, SLT_INT, SLT_VOID},
	{"SETMARGINSENSITIVEN", SLT_VOID, SCI_SETMARGINSENSITIVEN, SLT_INT, SLT_BOOL},
	{"GETMARGINSENSITIVEN", SLT_BOOL, SCI_GETMARGINSENSITIVEN, SLT_INT, SLT_VOID},
	{"SETMARGINCURSORN", SLT_VOID, SCI_SETMARGINCURSORN, SLT_INT, SLT_INT},
	{"GETMARGINCURSORN", SLT_INT, SCI_GETMARGINCURSORN, SLT_INT, SLT_VOID},
	{"SETMARGINBACKN", SLT_VOID, SCI_SETMARGINBACKN, SLT_INT, SLT_INT},
	{"GETMARGINBACKN", SLT_INT, SCI_GETMARGINBACKN, SLT_INT, SLT_VOID},
	{"SETMARGINS", SLT_VOID, SCI_SETMARGINS, SLT_INT, SLT_VOID},
	{"GETMARGINS", SLT_INT, SCI_GETMARGINS, SLT_VOID, SLT_VOID},
	{"STYLECLEARALL", SLT_VOID, SCI_STYLECLEARALL, SLT_VOID, SLT_VOID},
	{"STYLESETFORE", SLT_VOID, SCI_STYLESETFORE, SLT_INT, SLT_INT},
	{"STYLESETBACK", SLT_VOID, SCI_STYLESETBACK, SLT_INT, SLT_INT},
	{"STYLESETBOLD", SLT_VOID, SCI_STYLESETBOLD, SLT_INT, SLT_BOOL},
	{"STYLESETITALIC", SLT_VOID, SCI_STYLESETITALIC, SLT_INT, SLT_BOOL},
	{"STYLESETSIZE", SLT_VOID, SCI_STYLESETSIZE, SLT_INT, SLT_INT},
	{"STYLESETFONT", SLT_VOID, SCI_STYLESETFONT, SLT_INT, SLT_STRING},
	{"STYLESETEOLFILLED", SLT_VOID, SCI_STYLESETEOLFILLED, SLT_INT, SLT_BOOL},
	{"STYLERESETDEFAULT", SLT_VOID, SCI_STYLERESETDEFAULT, SLT_VOID, SLT_VOID},
	{"STYLESETUNDERLINE", SLT_VOID, SCI_STYLESETUNDERLINE, SLT_INT, SLT_BOOL},
	{"STYLEGETFORE", SLT_INT, SCI_STYLEGETFORE, SLT_INT, SLT_VOID},
	{"STYLEGETBACK", SLT_INT, SCI_STYLEGETBACK, SLT_INT, SLT_VOID},
	{"STYLEGETBOLD", SLT_BOOL, SCI_STYLEGETBOLD, SLT_INT, SLT_VOID},
	{"STYLEGETITALIC", SLT_BOOL, SCI_STYLEGETITALIC, SLT_INT, SLT_VOID},
	{"STYLEGETSIZE", SLT_INT, SCI_STYLEGETSIZE, SLT_INT, SLT_VOID},
	{"STYLEGETFONT", SLT_INT, SCI_STYLEGETFONT, SLT_INT, SLT_STRINGRESULT},
	{"STYLEGETEOLFILLED", SLT_BOOL, SCI_STYLEGETEOLFILLED, SLT_INT, SLT_VOID},
	{"STYLEGETUNDERLINE", SLT_BOOL, SCI_STYLEGETUNDERLINE, SLT_INT, SLT_VOID},
	{"STYLEGETCASE", SLT_INT, SCI_STYLEGETCASE, SLT_INT, SLT_VOID},
	{"STYLEGETCHARACTERSET", SLT_INT, SCI_STYLEGETCHARACTERSET, SLT_INT, SLT_VOID},
	{"STYLEGETVISIBLE", SLT_BOOL, SCI_STYLEGETVISIBLE, SLT_INT, SLT_VOID},
	{"STYLEGETCHANGEABLE", SLT_BOOL, SCI_STYLEGETCHANGEABLE, SLT_INT, SLT_VOID},
	{"STYLEGETHOTSPOT", SLT_BOOL, SCI_STYLEGETHOTSPOT, SLT_INT, SLT_VOID},
	{"STYLESETCASE", SLT_VOID, SCI_STYLESETCASE, SLT_INT, SLT_INT},
	{"STYLESETSIZEFRACTIONAL", SLT_VOID, SCI_STYLESETSIZEFRACTIONAL, SLT_INT, SLT_INT},
	{"STYLEGETSIZEFRACTIONAL", SLT_INT, SCI_STYLEGETSIZEFRACTIONAL, SLT_INT, SLT_VOID},
	{"STYLESETWEIGHT", SLT_VOID, SCI_STYLESETWEIGHT, SLT_INT, SLT_INT},
	{"STYLEGETWEIGHT", SLT_INT, SCI_STYLEGETWEIGHT, SLT_INT, SLT_VOID},
	{"STYLESETCHARACTERSET", SLT_VOID, SCI_STYLESETCHARACTERSET, SLT_INT, SLT_INT},
	{"STYLESETHOTSPOT", SLT_VOID, SCI_STYLESETHOTSPOT, SLT_INT, SLT_BOOL},
	{"SETSELFORE", SLT_VOID, SCI_SETSELFORE, SLT_BOOL, SLT_INT},
	{"SETSELBACK", SLT_VOID, SCI_SETSELBACK, SLT_BOOL, SLT_INT},
	{"GETSELALPHA", SLT_INT, SCI_GETSELALPHA, SLT_VOID, SLT_VOID},
	{"SETSELALPHA", SLT_VOID, SCI_SETSELALPHA, SLT_INT, SLT_VOID},
	{"GETSELEOLFILLED", SLT_BOOL, SCI_GETSELEOLFILLED, SLT_VOID, SLT_VOID},
	{"SETSELEOLFILLED", SLT_VOID, SCI_SETSELEOLFILLED, SLT_BOOL, SLT_VOID},
	{"SETCARETFORE", SLT_VOID, SCI_SETCARETFORE, SLT_INT, SLT_VOID},
	{"ASSIGNCMDKEY", SLT_VOID, SCI_ASSIGNCMDKEY, SLT_INT, SLT_INT},
	{"CLEARCMDKEY", SLT_VOID, SCI_CLEARCMDKEY, SLT_INT, SLT_VOID},
	{"CLEARALLCMDKEYS", SLT_VOID, SCI_CLEARALLCMDKEYS, SLT_VOID, SLT_VOID},
	{"SETSTYLINGEX", SLT_VOID, SCI_SETSTYLINGEX, SLT_INT, SLT_STRING},
	{"STYLESETVISIBLE", SLT_VOID, SCI_STYLESETVISIBLE, SLT_INT, SLT_BOOL},
	{"GETCARETPERIOD", SLT_INT, SCI_GETCARETPERIOD, SLT_VOID, SLT_VOID},
	{"SETCARETPERIOD", SLT_VOID, SCI_SETCARETPERIOD, SLT_INT, SLT_VOID},
	{"SETWORDCHARS", SLT_VOID, SCI_SETWORDCHARS, SLT_VOID, SLT_STRING},
	{"GETWORDCHARS", SLT_INT, SCI_GETWORDCHARS, SLT_VOID, SLT_STRINGRESULT},
	{"BEGINUNDOACTION", SLT_VOID, SCI_BEGINUNDOACTION, SLT_VOID, SLT_VOID},
	{"ENDUNDOACTION", SLT_VOID, SCI_ENDUNDOACTION, SLT_VOID, SLT_VOID},
	{"INDICSETSTYLE", SLT_VOID, SCI_INDICSETSTYLE, SLT_INT, SLT_INT},
	{"INDICGETSTYLE", SLT_INT, SCI_INDICGETSTYLE, SLT_INT, SLT_VOID},
	{"INDICSETFORE", SLT_VOID, SCI_INDICSETFORE, SLT_INT, SLT_INT},
	{"INDICGETFORE", SLT_INT, SCI_INDICGETFORE, SLT_INT, SLT_VOID},
	{"INDICSETUNDER", SLT_VOID, SCI_INDICSETUNDER, SLT_INT, SLT_BOOL},
	{"INDICGETUNDER", SLT_BOOL, SCI_INDICGETUNDER, SLT_INT, SLT_VOID},
	{"INDICSETHOVERSTYLE", SLT_VOID, SCI_INDICSETHOVERSTYLE, SLT_INT, SLT_INT},
	{"INDICGETHOVERSTYLE", SLT_INT, SCI_INDICGETHOVERSTYLE, SLT_INT, SLT_VOID},
	{"INDICSETHOVERFORE", SLT_VOID, SCI_INDICSETHOVERFORE, SLT_INT, SLT_INT},
	{"INDICGETHOVERFORE", SLT_INT, SCI_INDICGETHOVERFORE, SLT_INT, SLT_VOID},
	{"INDICSETFLAGS", SLT_VOID, SCI_INDICSETFLAGS, SLT_INT, SLT_INT},
	{"INDICGETFLAGS", SLT_INT, SCI_INDICGETFLAGS, SLT_INT, SLT_VOID},
	{"SETWHITESPACEFORE", SLT_VOID, SCI_SETWHITESPACEFORE, SLT_BOOL, SLT_INT},
	{"SETWHITESPACEBACK", SLT_VOID, SCI_SETWHITESPACEBACK, SLT_BOOL, SLT_INT},
	{"SETWHITESPACESIZE", SLT_VOID, SCI_SETWHITESPACESIZE, SLT_INT, SLT_VOID},
	{"GETWHITESPACESIZE", SLT_INT, SCI_GETWHITESPACESIZE, SLT_VOID, SLT_VOID},
	{"SETLINESTATE", SLT_VOID, SCI_SETLINESTATE, SLT_INT, SLT_INT},
	{"GETLINESTATE", SLT_INT, SCI_GETLINESTATE, SLT_INT, SLT_VOID},
	{"GETMAXLINESTATE", SLT_INT, SCI_GETMAXLINESTATE, SLT_VOID, SLT_VOID},
	{"GETCARETLINEVISIBLE", SLT_BOOL, SCI_GETCARETLINEVISIBLE, SLT_VOID, SLT_VOID},
	{"SETCARETLINEVISIBLE", SLT_VOID, SCI_SETCARETLINEVISIBLE, SLT_BOOL, SLT_VOID},
	{"GETCARETLINEBACK", SLT_INT, SCI_GETCARETLINEBACK, SLT_VOID, SLT_VOID},
	{"SETCARETLINEBACK", SLT_VOID, SCI_SETCARETLINEBACK, SLT_INT, SLT_VOID},
	{"GETCARETLINEFRAME", SLT_INT, SCI_GETCARETLINEFRAME, SLT_VOID, SLT_VOID},
	{"SETCARETLINEFRAME", SLT_VOID, SCI_SETCARETLINEFRAME, SLT_INT, SLT_VOID},
	{"STYLESETCHANGEABLE", SLT_VOID, SCI_STYLESETCHANGEABLE, SLT_INT, SLT_BOOL},
	{"AUTOCSHOW", SLT_VOID, SCI_AUTOCSHOW, SLT_INT, SLT_STRING},
	{"AUTOCCANCEL", SLT_VOID, SCI_AUTOCCANCEL, SLT_VOID, SLT_VOID},
	{"AUTOCACTIVE", SLT_BOOL, SCI_AUTOCACTIVE, SLT_VOID, SLT_VOID},
	{"AUTOCPOSSTART", SLT_INT, SCI_AUTOCPOSSTART, SLT_VOID, SLT_VOID},
	{"AUTOCCOMPLETE", SLT_VOID, SCI_AUTOCCOMPLETE, SLT_VOID, SLT_VOID},
	{"AUTOCSTOPS", SLT_VOID, SCI_AUTOCSTOPS, SLT_VOID, SLT_STRING},
	{"AUTOCSETSEPARATOR", SLT_VOID, SCI_AUTOCSETSEPARATOR, SLT_INT, SLT_VOID},
	{"AUTOCGETSEPARATOR", SLT_INT, SCI_AUTOCGETSEPARATOR, SLT_VOID, SLT_VOID},
	{"AUTOCSELECT", SLT_VOID, SCI_AUTOCSELECT, SLT_VOID, SLT_STRING},
	{"AUTOCSETCANCELATSTART", SLT_VOID, SCI_AUTOCSETCANCELATSTART, SLT_BOOL, SLT_VOID},
	{"AUTOCGETCANCELATSTART", SLT_BOOL, SCI_AUTOCGETCANCELATSTART, SLT_VOID, SLT_VOID},
	{"AUTOCSETFILLUPS", SLT_VOID, SCI_AUTOCSETFILLUPS, SLT_VOID, SLT_STRING},
	{"AUTOCSETCHOOSESINGLE", SLT_VOID, SCI_AUTOCSETCHOOSESINGLE, SLT_BOOL, SLT_VOID},
	{"AUTOCGETCHOOSESINGLE", SLT_BOOL, SCI_AUTOCGETCHOOSESINGLE, SLT_VOID, SLT_VOID},
	{"AUTOCSETIGNORECASE", SLT_VOID, SCI_AUTOCSETIGNORECASE, SLT_BOOL, SLT_VOID},
	{"AUTOCGETIGNORECASE", SLT_BOOL, SCI_AUTOCGETIGNORECASE, SLT_VOID, SLT_VOID},
	{"USERLISTSHOW", SLT_VOID, SCI_USERLISTSHOW, SLT_INT, SLT_STRING},
	{"AUTOCSETAUTOHIDE", SLT_VOID, SCI_AUTOCSETAUTOHIDE, SLT_BOOL, SLT_VOID},
	{"AUTOCGETAUTOHIDE", SLT_BOOL, SCI_AUTOCGETAUTOHIDE, SLT_VOID, SLT_VOID},
	{"AUTOCSETDROPRESTOFWORD", SLT_VOID, SCI_AUTOCSETDROPRESTOFWORD, SLT_BOOL, SLT_VOID},
	{"AUTOCGETDROPRESTOFWORD", SLT_BOOL, SCI_AUTOCGETDROPRESTOFWORD, SLT_VOID, SLT_VOID},
	{"REGISTERIMAGE", SLT_VOID, SCI_REGISTERIMAGE, SLT_INT, SLT_STRING},
	{"CLEARREGISTEREDIMAGES", SLT_VOID, SCI_CLEARREGISTEREDIMAGES, SLT_VOID, SLT_VOID},
	{"AUTOCGETTYPESEPARATOR", SLT_INT, SCI_AUTOCGETTYPESEPARATOR, SLT_VOID, SLT_VOID},
	{"AUTOCSETTYPESEPARATOR", SLT_VOID, SCI_AUTOCSETTYPESEPARATOR, SLT_INT, SLT_VOID},
	{"AUTOCSETMAXWIDTH", SLT_VOID, SCI_AUTOCSETMAXWIDTH, SLT_INT, SLT_VOID},
	{"AUTOCGETMAXWIDTH", SLT_INT, SCI_AUTOCGETMAXWIDTH, SLT_VOID, SLT_VOID},
	{"AUTOCSETMAXHEIGHT", SLT_VOID, SCI_AUTOCSETMAXHEIGHT, SLT_INT, SLT_VOID},
	{"AUTOCGETMAXHEIGHT", SLT_INT, SCI_AUTOCGETMAXHEIGHT, SLT_VOID, SLT_VOID},
	{"SETINDENT", SLT_VOID, SCI_SETINDENT, SLT_INT, SLT_VOID},
	{"GETINDENT", SLT_INT, SCI_GETINDENT, SLT_VOID, SLT_VOID},
	{"SETUSETABS", SLT_VOID, SCI_SETUSETABS, SLT_BOOL, SLT_VOID},
	{"GETUSETABS", SLT_BOOL, SCI_GETUSETABS, SLT_VOID, SLT_VOID},
	{"SETLINEINDENTATION", SLT_VOID, SCI_SETLINEINDENTATION, SLT_INT, SLT_INT},
	{"GETLINEINDENTATION", SLT_INT, SCI_GETLINEINDENTATION, SLT_INT, SLT_VOID},
	{"GETLINEINDENTPOSITION", SLT_INT, SCI_GETLINEINDENTPOSITION, SLT_INT, SLT_VOID},
	{"GETCOLUMN", SLT_INT, SCI_GETCOLUMN, SLT_INT, SLT_VOID},
	{"COUNTCHARACTERS", SLT_INT, SCI_COUNTCHARACTERS, SLT_INT, SLT_INT},
	{"COUNTCODEUNITS", SLT_INT, SCI_COUNTCODEUNITS, SLT_INT, SLT_INT},
	{"SETHSCROLLBAR", SLT_VOID, SCI_SETHSCROLLBAR, SLT_BOOL, SLT_VOID},
	{"GETHSCROLLBAR", SLT_BOOL, SCI_GETHSCROLLBAR, SLT_VOID, SLT_VOID},
	{"SETINDENTATIONGUIDES", SLT_VOID, SCI_SETINDENTATIONGUIDES, SLT_INT, SLT_VOID},
	{"GETINDENTATIONGUIDES", SLT_INT, SCI_GETINDENTATIONGUIDES, SLT_VOID, SLT_VOID},
	{"SETHIGHLIGHTGUIDE", SLT_VOID, SCI_SETHIGHLIGHTGUIDE, SLT_INT, SLT_VOID},
	{"GETHIGHLIGHTGUIDE", SLT_INT, SCI_GETHIGHLIGHTGUIDE, SLT_VOID, SLT_VOID},
	{"GETLINEENDPOSITION", SLT_INT, SCI_GETLINEENDPOSITION, SLT_INT, SLT_VOID},
	{"GETCODEPAGE", SLT_INT, SCI_GETCODEPAGE, SLT_VOID, SLT_VOID},
	{"GETCARETFORE", SLT_INT, SCI_GETCARETFORE, SLT_VOID, SLT_VOID},
	{"GETREADONLY", SLT_BOOL, SCI_GETREADONLY, SLT_VOID, SLT_VOID},
	{"SETCURRENTPOS", SLT_VOID, SCI_SETCURRENTPOS, SLT_INT, SLT_VOID},
	{"SETSELECTIONSTART", SLT_VOID, SCI_SETSELECTIONSTART, SLT_INT, SLT_VOID},
	{"GETSELECTIONSTART", SLT_INT, SCI_GETSELECTIONSTART, SLT_VOID, SLT_VOID},
	{"SETSELECTIONEND", SLT_VOID, SCI_SETSELECTIONEND, SLT_INT, SLT_VOID},
	{"GETSELECTIONEND", SLT_INT, SCI_GETSELECTIONEND, SLT_VOID, SLT_VOID},
	{"SETEMPTYSELECTION", SLT_VOID, SCI_SETEMPTYSELECTION, SLT_INT, SLT_VOID},
	{"SETPRINTMAGNIFICATION", SLT_VOID, SCI_SETPRINTMAGNIFICATION, SLT_INT, SLT_VOID},
	{"GETPRINTMAGNIFICATION", SLT_INT, SCI_GETPRINTMAGNIFICATION, SLT_VOID, SLT_VOID},
	{"SETPRINTCOLOURMODE", SLT_VOID, SCI_SETPRINTCOLOURMODE, SLT_INT, SLT_VOID},
	{"GETPRINTCOLOURMODE", SLT_INT, SCI_GETPRINTCOLOURMODE, SLT_VOID, SLT_VOID},
	{"FINDTEXT", SLT_INT, SCI_FINDTEXT, SLT_INT, SLT_FINDTEXT},
	{"FORMATRANGE", SLT_INT, SCI_FORMATRANGE, SLT_BOOL, SLT_FORMATRANGE},
	{"GETFIRSTVISIBLELINE", SLT_INT, SCI_GETFIRSTVISIBLELINE, SLT_VOID, SLT_VOID},
	{"GETLINE", SLT_INT, SCI_GETLINE, SLT_INT, SLT_STRINGRESULT},
	{"GETLINECOUNT", SLT_INT, SCI_GETLINECOUNT, SLT_VOID, SLT_VOID},
	{"SETMARGINLEFT", SLT_VOID, SCI_SETMARGINLEFT, SLT_VOID, SLT_INT},
	{"GETMARGINLEFT", SLT_INT, SCI_GETMARGINLEFT, SLT_VOID, SLT_VOID},
	{"SETMARGINRIGHT", SLT_VOID, SCI_SETMARGINRIGHT, SLT_VOID, SLT_INT},
	{"GETMARGINRIGHT", SLT_INT, SCI_GETMARGINRIGHT, SLT_VOID, SLT_VOID},
	{"GETMODIFY", SLT_BOOL, SCI_GETMODIFY, SLT_VOID, SLT_VOID},
	{"SETSEL", SLT_VOID, SCI_SETSEL, SLT_INT, SLT_INT},
	{"GETSELTEXT", SLT_INT, SCI_GETSELTEXT, SLT_VOID, SLT_STRINGRESULT},
	{"GETTEXTRANGE", SLT_INT, SCI_GETTEXTRANGE, SLT_VOID, SLT_TEXTRANGE},
	{"HIDESELECTION", SLT_VOID, SCI_HIDESELECTION, SLT_BOOL, SLT_VOID},
	{"POINTXFROMPOSITION", SLT_INT, SCI_POINTXFROMPOSITION, SLT_VOID, SLT_INT},
	{"POINTYFROMPOSITION", SLT_INT, SCI_POINTYFROMPOSITION, SLT_VOID, SLT_INT},
	{"LINEFROMPOSITION", SLT_INT, SCI_LINEFROMPOSITION, SLT_INT, SLT_VOID},
	{"POSITIONFROMLINE", SLT_INT, SCI_POSITIONFROMLINE, SLT_INT, SLT_VOID},
	{"LINESCROLL", SLT_VOID, SCI_LINESCROLL, SLT_INT, SLT_INT},
	{"SCROLLCARET", SLT_VOID, SCI_SCROLLCARET, SLT_VOID, SLT_VOID},
	{"SCROLLRANGE", SLT_VOID, SCI_SCROLLRANGE, SLT_INT, SLT_INT},
	{"REPLACESEL", SLT_VOID, SCI_REPLACESEL, SLT_VOID, SLT_STRING},
	{"SETREADONLY", SLT_VOID, SCI_SETREADONLY, SLT_BOOL, SLT_VOID},
	{"NULL", SLT_VOID, SCI_NULL, SLT_VOID, SLT_VOID},
	{"CANPASTE", SLT_BOOL, SCI_CANPASTE, SLT_VOID, SLT_VOID},
	{"CANUNDO", SLT_BOOL, SCI_CANUNDO, SLT_VOID, SLT_VOID},
	{"EMPTYUNDOBUFFER", SLT_VOID, SCI_EMPTYUNDOBUFFER, SLT_VOID, SLT_VOID},
	{"UNDO", SLT_VOID, SCI_UNDO, SLT_VOID, SLT_VOID},
	{"CUT", SLT_VOID, SCI_CUT, SLT_VOID, SLT_VOID},
	{"COPY", SLT_VOID, SCI_COPY, SLT_VOID, SLT_VOID},
	{"PASTE", SLT_VOID, SCI_PASTE, SLT_VOID, SLT_VOID},
	{"CLEAR", SLT_VOID, SCI_CLEAR, SLT_VOID, SLT_VOID},
	{"SETTEXT", SLT_VOID, SCI_SETTEXT, SLT_VOID, SLT_STRING},
	{"GETTEXT", SLT_INT, SCI_GETTEXT, SLT_INT, SLT_STRINGRESULT},
	{"GETTEXTLENGTH", SLT_INT, SCI_GETTEXTLENGTH, SLT_VOID, SLT_VOID},
	{"GETDIRECTFUNCTION", SLT_INT, SCI_GETDIRECTFUNCTION, SLT_VOID, SLT_VOID},
	{"GETDIRECTPOINTER", SLT_INT, SCI_GETDIRECTPOINTER, SLT_VOID, SLT_VOID},
	{"SETOVERTYPE", SLT_VOID, SCI_SETOVERTYPE, SLT_BOOL, SLT_VOID},
	{"GETOVERTYPE", SLT_BOOL, SCI_GETOVERTYPE, SLT_VOID, SLT_VOID},
	{"SETCARETWIDTH", SLT_VOID, SCI_SETCARETWIDTH, SLT_INT, SLT_VOID},
	{"GETCARETWIDTH", SLT_INT, SCI_GETCARETWIDTH, SLT_VOID, SLT_VOID},
	{"SETTARGETSTART", SLT_VOID, SCI_SETTARGETSTART, SLT_INT, SLT_VOID},
	{"GETTARGETSTART", SLT_INT, SCI_GETTARGETSTART, SLT_VOID, SLT_VOID},
	{"SETTARGETEND", SLT_VOID, SCI_SETTARGETEND, SLT_INT, SLT_VOID},
	{"GETTARGETEND", SLT_INT, SCI_GETTARGETEND, SLT_VOID, SLT_VOID},
	{"SETTARGETRANGE", SLT_VOID, SCI_SETTARGETRANGE, SLT_INT, SLT_INT},
	{"GETTARGETTEXT", SLT_INT, SCI_GETTARGETTEXT, SLT_VOID, SLT_STRINGRESULT},
	{"TARGETFROMSELECTION", SLT_VOID, SCI_TARGETFROMSELECTION, SLT_VOID, SLT_VOID},
	{"TARGETWHOLEDOCUMENT", SLT_VOID, SCI_TARGETWHOLEDOCUMENT, SLT_VOID, SLT_VOID},
	{"REPLACETARGET", SLT_INT, SCI_REPLACETARGET, SLT_INT, SLT_STRING},
	{"REPLACETARGETRE", SLT_INT, SCI_REPLACETARGETRE, SLT_INT, SLT_STRING},
	{"SEARCHINTARGET", SLT_INT, SCI_SEARCHINTARGET, SLT_INT, SLT_STRING},
	{"SETSEARCHFLAGS", SLT_VOID, SCI_SETSEARCHFLAGS, SLT_INT, SLT_VOID},
	{"GETSEARCHFLAGS", SLT_INT, SCI_GETSEARCHFLAGS, SLT_VOID, SLT_VOID},
	{"CALLTIPSHOW", SLT_VOID, SCI_CALLTIPSHOW, SLT_INT, SLT_STRING},
	{"CALLTIPCANCEL", SLT_VOID, SCI_CALLTIPCANCEL, SLT_VOID, SLT_VOID},
	{"CALLTIPACTIVE", SLT_BOOL, SCI_CALLTIPACTIVE, SLT_VOID, SLT_VOID},
	{"CALLTIPPOSSTART", SLT_INT, SCI_CALLTIPPOSSTART, SLT_VOID, SLT_VOID},
	{"CALLTIPSETPOSSTART", SLT_VOID, SCI_CALLTIPSETPOSSTART, SLT_INT, SLT_VOID},
	{"CALLTIPSETHLT", SLT_VOID, SCI_CALLTIPSETHLT, SLT_INT, SLT_INT},
	{"CALLTIPSETBACK", SLT_VOID, SCI_CALLTIPSETBACK, SLT_INT, SLT_VOID},
	{"CALLTIPSETFORE", SLT_VOID, SCI_CALLTIPSETFORE, SLT_INT, SLT_VOID},
	{"CALLTIPSETFOREHLT", SLT_VOID, SCI_CALLTIPSETFOREHLT, SLT_INT, SLT_VOID},
	{"CALLTIPUSESTYLE", SLT_VOID, SCI_CALLTIPUSESTYLE, SLT_INT, SLT_VOID},
	{"CALLTIPSETPOSITION", SLT_VOID, SCI_CALLTIPSETPOSITION, SLT_BOOL, SLT_VOID},
	{"VISIBLEFROMDOCLINE", SLT_INT, SCI_VISIBLEFROMDOCLINE, SLT_INT, SLT_VOID},
	{"DOCLINEFROMVISIBLE", SLT_INT, SCI_DOCLINEFROMVISIBLE, SLT_INT, SLT_VOID},
	{"WRAPCOUNT", SLT_INT, SCI_WRAPCOUNT, SLT_INT, SLT_VOID},
	{"SETFOLDLEVEL", SLT_VOID, SCI_SETFOLDLEVEL, SLT_INT, SLT_INT},
	{"GETFOLDLEVEL", SLT_INT, SCI_GETFOLDLEVEL, SLT_INT, SLT_VOID},
	{"GETLASTCHILD", SLT_INT, SCI_GETLASTCHILD, SLT_INT, SLT_INT},
	{"GETFOLDPARENT", SLT_INT, SCI_GETFOLDPARENT, SLT_INT, SLT_VOID},
	{"SHOWLINES", SLT_VOID, SCI_SHOWLINES, SLT_INT, SLT_INT},
	{"HIDELINES", SLT_VOID, SCI_HIDELINES, SLT_INT, SLT_INT},
	{"GETLINEVISIBLE", SLT_BOOL, SCI_GETLINEVISIBLE, SLT_INT, SLT_VOID},
	{"GETALLLINESVISIBLE", SLT_BOOL, SCI_GETALLLINESVISIBLE, SLT_VOID, SLT_VOID},
	{"SETFOLDEXPANDED", SLT_VOID, SCI_SETFOLDEXPANDED, SLT_INT, SLT_BOOL},
	{"GETFOLDEXPANDED", SLT_BOOL, SCI_GETFOLDEXPANDED, SLT_INT, SLT_VOID},
	{"TOGGLEFOLD", SLT_VOID, SCI_TOGGLEFOLD, SLT_INT, SLT_VOID},
	{"TOGGLEFOLDSHOWTEXT", SLT_VOID, SCI_TOGGLEFOLDSHOWTEXT, SLT_INT, SLT_STRING},
	{"FOLDDISPLAYTEXTSETSTYLE", SLT_VOID, SCI_FOLDDISPLAYTEXTSETSTYLE, SLT_INT, SLT_VOID},
	{"FOLDLINE", SLT_VOID, SCI_FOLDLINE, SLT_INT, SLT_INT},
	{"FOLDCHILDREN", SLT_VOID, SCI_FOLDCHILDREN, SLT_INT, SLT_INT},
	{"EXPANDCHILDREN", SLT_VOID, SCI_EXPANDCHILDREN, SLT_INT, SLT_INT},
	{"FOLDALL", SLT_VOID, SCI_FOLDALL, SLT_INT, SLT_VOID},
	{"ENSUREVISIBLE", SLT_VOID, SCI_ENSUREVISIBLE, SLT_INT, SLT_VOID},
	{"SETAUTOMATICFOLD", SLT_VOID, SCI_SETAUTOMATICFOLD, SLT_INT, SLT_VOID},
	{"GETAUTOMATICFOLD", SLT_INT, SCI_GETAUTOMATICFOLD, SLT_VOID, SLT_VOID},
	{"SETFOLDFLAGS", SLT_VOID, SCI_SETFOLDFLAGS, SLT_INT, SLT_VOID},
	{"ENSUREVISIBLEENFORCEPOLICY", SLT_VOID, SCI_ENSUREVISIBLEENFORCEPOLICY, SLT_INT, SLT_VOID},
	{"SETTABINDENTS", SLT_VOID, SCI_SETTABINDENTS, SLT_BOOL, SLT_VOID},
	{"GETTABINDENTS", SLT_BOOL, SCI_GETTABINDENTS, SLT_VOID, SLT_VOID},
	{"SETBACKSPACEUNINDENTS", SLT_VOID, SCI_SETBACKSPACEUNINDENTS, SLT_BOOL, SLT_VOID},
	{"GETBACKSPACEUNINDENTS", SLT_BOOL, SCI_GETBACKSPACEUNINDENTS, SLT_VOID, SLT_VOID},
	{"SETMOUSEDWELLTIME", SLT_VOID, SCI_SETMOUSEDWELLTIME, SLT_INT, SLT_VOID},
	{"GETMOUSEDWELLTIME", SLT_INT, SCI_GETMOUSEDWELLTIME, SLT_VOID, SLT_VOID},
	{"WORDSTARTPOSITION", SLT_INT, SCI_WORDSTARTPOSITION, SLT_INT, SLT_BOOL},
	{"WORDENDPOSITION", SLT_INT, SCI_WORDENDPOSITION, SLT_INT, SLT_BOOL},
	{"ISRANGEWORD", SLT_BOOL, SCI_ISRANGEWORD, SLT_INT, SLT_INT},
	{"SETIDLESTYLING", SLT_VOID, SCI_SETIDLESTYLING, SLT_INT, SLT_VOID},
	{"GETIDLESTYLING", SLT_INT, SCI_GETIDLESTYLING, SLT_VOID, SLT_VOID},
	{"SETWRAPMODE", SLT_VOID, SCI_SETWRAPMODE, SLT_INT, SLT_VOID},
	{"GETWRAPMODE", SLT_INT, SCI_GETWRAPMODE, SLT_VOID, SLT_VOID},
	{"SETWRAPVISUALFLAGS", SLT_VOID, SCI_SETWRAPVISUALFLAGS, SLT_INT, SLT_VOID},
	{"GETWRAPVISUALFLAGS", SLT_INT, SCI_GETWRAPVISUALFLAGS, SLT_VOID, SLT_VOID},
	{"SETWRAPVISUALFLAGSLOCATION", SLT_VOID, SCI_SETWRAPVISUALFLAGSLOCATION, SLT_INT, SLT_VOID},
	{"GETWRAPVISUALFLAGSLOCATION", SLT_INT, SCI_GETWRAPVISUALFLAGSLOCATION, SLT_VOID, SLT_VOID},
	{"SETWRAPSTARTINDENT", SLT_VOID, SCI_SETWRAPSTARTINDENT, SLT_INT, SLT_VOID},
	{"GETWRAPSTARTINDENT", SLT_INT, SCI_GETWRAPSTARTINDENT, SLT_VOID, SLT_VOID},
	{"SETWRAPINDENTMODE", SLT_VOID, SCI_SETWRAPINDENTMODE, SLT_INT, SLT_VOID},
	{"GETWRAPINDENTMODE", SLT_INT, SCI_GETWRAPINDENTMODE, SLT_VOID, SLT_VOID},
	{"SETLAYOUTCACHE", SLT_VOID, SCI_SETLAYOUTCACHE, SLT_INT, SLT_VOID},
	{"GETLAYOUTCACHE", SLT_INT, SCI_GETLAYOUTCACHE, SLT_VOID, SLT_VOID},
	{"SETSCROLLWIDTH", SLT_VOID, SCI_SETSCROLLWIDTH, SLT_INT, SLT_VOID},
	{"GETSCROLLWIDTH", SLT_INT, SCI_GETSCROLLWIDTH, SLT_VOID, SLT_VOID},
	{"SETSCROLLWIDTHTRACKING", SLT_VOID, SCI_SETSCROLLWIDTHTRACKING, SLT_BOOL, SLT_VOID},
	{"GETSCROLLWIDTHTRACKING", SLT_BOOL, SCI_GETSCROLLWIDTHTRACKING, SLT_VOID, SLT_VOID},
	{"TEXTWIDTH", SLT_INT, SCI_TEXTWIDTH, SLT_INT, SLT_STRING},
	{"SETENDATLASTLINE", SLT_VOID, SCI_SETENDATLASTLINE, SLT_BOOL, SLT_VOID},
	{"GETENDATLASTLINE", SLT_BOOL, SCI_GETENDATLASTLINE, SLT_VOID, SLT_VOID},
	{"TEXTHEIGHT", SLT_INT, SCI_TEXTHEIGHT, SLT_INT, SLT_VOID},
	{"SETVSCROLLBAR", SLT_VOID, SCI_SETVSCROLLBAR, SLT_BOOL, SLT_VOID},
	{"GETVSCROLLBAR", SLT_BOOL, SCI_GETVSCROLLBAR, SLT_VOID, SLT_VOID},
	{"APPENDTEXT", SLT_VOID, SCI_APPENDTEXT, SLT_INT, SLT_STRING},
	{"GETTWOPHASEDRAW", SLT_BOOL, SCI_GETTWOPHASEDRAW, SLT_VOID, SLT_VOID},
	{"SETTWOPHASEDRAW", SLT_VOID, SCI_SETTWOPHASEDRAW, SLT_BOOL, SLT_VOID},
	{"GETPHASESDRAW", SLT_INT, SCI_GETPHASESDRAW, SLT_VOID, SLT_VOID},
	{"SETPHASESDRAW", SLT_VOID, SCI_SETPHASESDRAW, SLT_INT, SLT_VOID},
	{"SETFONTQUALITY", SLT_VOID, SCI_SETFONTQUALITY, SLT_INT, SLT_VOID},
	{"GETFONTQUALITY", SLT_INT, SCI_GETFONTQUALITY, SLT_VOID, SLT_VOID},
	{"SETFIRSTVISIBLELINE", SLT_VOID, SCI_SETFIRSTVISIBLELINE, SLT_INT, SLT_VOID},
	{"SETMULTIPASTE", SLT_VOID, SCI_SETMULTIPASTE, SLT_INT, SLT_VOID},
	{"GETMULTIPASTE", SLT_INT, SCI_GETMULTIPASTE, SLT_VOID, SLT_VOID},
	{"GETTAG", SLT_INT, SCI_GETTAG, SLT_INT, SLT_STRINGRESULT},
	{"LINESJOIN", SLT_VOID, SCI_LINESJOIN, SLT_VOID, SLT_VOID},
	{"LINESSPLIT", SLT_VOID, SCI_LINESSPLIT, SLT_INT, SLT_VOID},
	{"SETFOLDMARGINCOLOUR", SLT_VOID, SCI_SETFOLDMARGINCOLOUR, SLT_BOOL, SLT_INT},
	{"SETFOLDMARGINHICOLOUR", SLT_VOID, SCI_SETFOLDMARGINHICOLOUR, SLT_BOOL, SLT_INT},
	{"SETACCESSIBILITY", SLT_VOID, SCI_SETACCESSIBILITY, SLT_INT, SLT_VOID},
	{"GETACCESSIBILITY", SLT_INT, SCI_GETACCESSIBILITY, SLT_VOID, SLT_VOID},
	{"LINEDOWN", SLT_VOID, SCI_LINEDOWN, SLT_VOID, SLT_VOID},
	{"LINEDOWNEXTEND", SLT_VOID, SCI_LINEDOWNEXTEND, SLT_VOID, SLT_VOID},
	{"LINEUP", SLT_VOID, SCI_LINEUP, SLT_VOID, SLT_VOID},
	{"LINEUPEXTEND", SLT_VOID, SCI_LINEUPEXTEND, SLT_VOID, SLT_VOID},
	{"CHARLEFT", SLT_VOID, SCI_CHARLEFT, SLT_VOID, SLT_VOID},
	{"CHARLEFTEXTEND", SLT_VOID, SCI_CHARLEFTEXTEND, SLT_VOID, SLT_VOID},
	{"CHARRIGHT", SLT_VOID, SCI_CHARRIGHT, SLT_VOID, SLT_VOID},
	{"CHARRIGHTEXTEND", SLT_VOID, SCI_CHARRIGHTEXTEND, SLT_VOID, SLT_VOID},
	{"WORDLEFT", SLT_VOID, SCI_WORDLEFT, SLT_VOID, SLT_VOID},
	{"WORDLEFTEXTEND", SLT_VOID, SCI_WORDLEFTEXTEND, SLT_VOID, SLT_VOID},
	{"WORDRIGHT", SLT_VOID, SCI_WORDRIGHT, SLT_VOID, SLT_VOID},
	{"WORDRIGHTEXTEND", SLT_VOID, SCI_WORDRIGHTEXTEND, SLT_VOID, SLT_VOID},
	{"HOME", SLT_VOID, SCI_HOME, SLT_VOID, SLT_VOID},
	{"HOMEEXTEND", SLT_VOID, SCI_HOMEEXTEND, SLT_VOID, SLT_VOID},
	{"LINEEND", SLT_VOID, SCI_LINEEND, SLT_VOID, SLT_VOID},
	{"LINEENDEXTEND", SLT_VOID, SCI_LINEENDEXTEND, SLT_VOID, SLT_VOID},
	{"DOCUMENTSTART", SLT_VOID, SCI_DOCUMENTSTART, SLT_VOID, SLT_VOID},
	{"DOCUMENTSTARTEXTEND", SLT_VOID, SCI_DOCUMENTSTARTEXTEND, SLT_VOID, SLT_VOID},
	{"DOCUMENTEND", SLT_VOID, SCI_DOCUMENTEND, SLT_VOID, SLT_VOID},
	{"DOCUMENTENDEXTEND", SLT_VOID, SCI_DOCUMENTENDEXTEND, SLT_VOID, SLT_VOID},
	{"PAGEUP", SLT_VOID, SCI_PAGEUP, SLT_VOID, SLT_VOID},
	{"PAGEUPEXTEND", SLT_VOID, SCI_PAGEUPEXTEND, SLT_VOID, SLT_VOID},
	{"PAGEDOWN", SLT_VOID, SCI_PAGEDOWN, SLT_VOID, SLT_VOID},
	{"PAGEDOWNEXTEND", SLT_VOID, SCI_PAGEDOWNEXTEND, SLT_VOID, SLT_VOID},
	{"EDITTOGGLEOVERTYPE", SLT_VOID, SCI_EDITTOGGLEOVERTYPE, SLT_VOID, SLT_VOID},
	{"CANCEL", SLT_VOID, SCI_CANCEL, SLT_VOID, SLT_VOID},
	{"DELETEBACK", SLT_VOID, SCI_DELETEBACK, SLT_VOID, SLT_VOID},
	{"TAB", SLT_VOID, SCI_TAB, SLT_VOID, SLT_VOID},
	{"BACKTAB", SLT_VOID, SCI_BACKTAB, SLT_VOID, SLT_VOID},
	{"NEWLINE", SLT_VOID, SCI_NEWLINE, SLT_VOID, SLT_VOID},
	{"FORMFEED", SLT_VOID, SCI_FORMFEED, SLT_VOID, SLT_VOID},
	{"VCHOME", SLT_VOID, SCI_VCHOME, SLT_VOID, SLT_VOID},
	{"VCHOMEEXTEND", SLT_VOID, SCI_VCHOMEEXTEND, SLT_VOID, SLT_VOID},
	{"ZOOMIN", SLT_VOID, SCI_ZOOMIN, SLT_VOID, SLT_VOID},
	{"ZOOMOUT", SLT_VOID, SCI_ZOOMOUT, SLT_VOID, SLT_VOID},
	{"DELWORDLEFT", SLT_VOID, SCI_DELWORDLEFT, SLT_VOID, SLT_VOID},
	{"DELWORDRIGHT", SLT_VOID, SCI_DELWORDRIGHT, SLT_VOID, SLT_VOID},
	{"DELWORDRIGHTEND", SLT_VOID, SCI_DELWORDRIGHTEND, SLT_VOID, SLT_VOID},
	{"LINECUT", SLT_VOID, SCI_LINECUT, SLT_VOID, SLT_VOID},
	{"LINEDELETE", SLT_VOID, SCI_LINEDELETE, SLT_VOID, SLT_VOID},
	{"LINETRANSPOSE", SLT_VOID, SCI_LINETRANSPOSE, SLT_VOID, SLT_VOID},
	{"LINEREVERSE", SLT_VOID, SCI_LINEREVERSE, SLT_VOID, SLT_VOID},
	{"LINEDUPLICATE", SLT_VOID, SCI_LINEDUPLICATE, SLT_VOID, SLT_VOID},
	{"LOWERCASE", SLT_VOID, SCI_LOWERCASE, SLT_VOID, SLT_VOID},
	{"UPPERCASE", SLT_VOID, SCI_UPPERCASE, SLT_VOID, SLT_VOID},
	{"LINESCROLLDOWN", SLT_VOID, SCI_LINESCROLLDOWN, SLT_VOID, SLT_VOID},
	{"LINESCROLLUP", SLT_VOID, SCI_LINESCROLLUP, SLT_VOID, SLT_VOID},
	{"DELETEBACKNOTLINE", SLT_VOID, SCI_DELETEBACKNOTLINE, SLT_VOID, SLT_VOID},
	{"HOMEDISPLAY", SLT_VOID, SCI_HOMEDISPLAY, SLT_VOID, SLT_VOID},
	{"HOMEDISPLAYEXTEND", SLT_VOID, SCI_HOMEDISPLAYEXTEND, SLT_VOID, SLT_VOID},
	{"LINEENDDISPLAY", SLT_VOID, SCI_LINEENDDISPLAY, SLT_VOID, SLT_VOID},
	{"LINEENDDISPLAYEXTEND", SLT_VOID, SCI_LINEENDDISPLAYEXTEND, SLT_VOID, SLT_VOID},
	{"HOMEWRAP", SLT_VOID, SCI_HOMEWRAP, SLT_VOID, SLT_VOID},
	{"HOMEWRAPEXTEND", SLT_VOID, SCI_HOMEWRAPEXTEND, SLT_VOID, SLT_VOID},
	{"LINEENDWRAP", SLT_VOID, SCI_LINEENDWRAP, SLT_VOID, SLT_VOID},
	{"LINEENDWRAPEXTEND", SLT_VOID, SCI_LINEENDWRAPEXTEND, SLT_VOID, SLT_VOID},
	{"VCHOMEWRAP", SLT_VOID, SCI_VCHOMEWRAP, SLT_VOID, SLT_VOID},
	{"VCHOMEWRAPEXTEND", SLT_VOID, SCI_VCHOMEWRAPEXTEND, SLT_VOID, SLT_VOID},
	{"LINECOPY", SLT_VOID, SCI_LINECOPY, SLT_VOID, SLT_VOID},
	{"MOVECARETINSIDEVIEW", SLT_VOID, SCI_MOVECARETINSIDEVIEW, SLT_VOID, SLT_VOID},
	{"LINELENGTH", SLT_INT, SCI_LINELENGTH, SLT_INT, SLT_VOID},
	{"BRACEHIGHLIGHT", SLT_VOID, SCI_BRACEHIGHLIGHT, SLT_INT, SLT_INT},
	{"BRACEHIGHLIGHTINDICATOR", SLT_VOID, SCI_BRACEHIGHLIGHTINDICATOR, SLT_BOOL, SLT_INT},
	{"BRACEBADLIGHT", SLT_VOID, SCI_BRACEBADLIGHT, SLT_INT, SLT_VOID},
	{"BRACEBADLIGHTINDICATOR", SLT_VOID, SCI_BRACEBADLIGHTINDICATOR, SLT_BOOL, SLT_INT},
	{"BRACEMATCH", SLT_INT, SCI_BRACEMATCH, SLT_INT, SLT_INT},
	{"GETVIEWEOL", SLT_BOOL, SCI_GETVIEWEOL, SLT_VOID, SLT_VOID},
	{"SETVIEWEOL", SLT_VOID, SCI_SETVIEWEOL, SLT_BOOL, SLT_VOID},
	{"GETDOCPOINTER", SLT_INT, SCI_GETDOCPOINTER, SLT_VOID, SLT_VOID},
	{"SETDOCPOINTER", SLT_VOID, SCI_SETDOCPOINTER, SLT_VOID, SLT_INT},
	{"SETMODEVENTMASK", SLT_VOID, SCI_SETMODEVENTMASK, SLT_INT, SLT_VOID},
	{"GETEDGECOLUMN", SLT_INT, SCI_GETEDGECOLUMN, SLT_VOID, SLT_VOID},
	{"SETEDGECOLUMN", SLT_VOID, SCI_SETEDGECOLUMN, SLT_INT, SLT_VOID},
	{"GETEDGEMODE", SLT_INT, SCI_GETEDGEMODE, SLT_VOID, SLT_VOID},
	{"SETEDGEMODE", SLT_VOID, SCI_SETEDGEMODE, SLT_INT, SLT_VOID},
	{"GETEDGECOLOUR", SLT_INT, SCI_GETEDGECOLOUR, SLT_VOID, SLT_VOID},
	{"SETEDGECOLOUR", SLT_VOID, SCI_SETEDGECOLOUR, SLT_INT, SLT_VOID},
	{"MULTIEDGEADDLINE", SLT_VOID, SCI_MULTIEDGEADDLINE, SLT_INT, SLT_INT},
	{"MULTIEDGECLEARALL", SLT_VOID, SCI_MULTIEDGECLEARALL, SLT_VOID, SLT_VOID},
	{"SEARCHANCHOR", SLT_VOID, SCI_SEARCHANCHOR, SLT_VOID, SLT_VOID},
	{"SEARCHNEXT", SLT_INT, SCI_SEARCHNEXT, SLT_INT, SLT_STRING},
	{"SEARCHPREV", SLT_INT, SCI_SEARCHPREV, SLT_INT, SLT_STRING},
	{"LINESONSCREEN", SLT_INT, SCI_LINESONSCREEN, SLT_VOID, SLT_VOID},
	{"USEPOPUP", SLT_VOID, SCI_USEPOPUP, SLT_INT, SLT_VOID},
	{"SELECTIONISRECTANGLE", SLT_BOOL, SCI_SELECTIONISRECTANGLE, SLT_VOID, SLT_VOID},
	{"SETZOOM", SLT_VOID, SCI_SETZOOM, SLT_INT, SLT_VOID},
	{"GETZOOM", SLT_INT, SCI_GETZOOM, SLT_VOID, SLT_VOID},
	{"CREATEDOCUMENT", SLT_INT, SCI_CREATEDOCUMENT, SLT_INT, SLT_INT},
	{"ADDREFDOCUMENT", SLT_VOID, SCI_ADDREFDOCUMENT, SLT_VOID, SLT_INT},
	{"RELEASEDOCUMENT", SLT_VOID, SCI_RELEASEDOCUMENT, SLT_VOID, SLT_INT},
	{"GETDOCUMENTOPTIONS", SLT_INT, SCI_GETDOCUMENTOPTIONS, SLT_VOID, SLT_VOID},
	{"GETMODEVENTMASK", SLT_INT, SCI_GETMODEVENTMASK, SLT_VOID, SLT_VOID},
	{"SETCOMMANDEVENTS", SLT_VOID, SCI_SETCOMMANDEVENTS, SLT_BOOL, SLT_VOID},
	{"GETCOMMANDEVENTS", SLT_BOOL, SCI_GETCOMMANDEVENTS, SLT_VOID, SLT_VOID},
	{"SETFOCUS", SLT_VOID, SCI_SETFOCUS, SLT_BOOL, SLT_VOID},
	{"GETFOCUS", SLT_BOOL, SCI_GETFOCUS, SLT_VOID, SLT_VOID},
	{"SETSTATUS", SLT_VOID, SCI_SETSTATUS, SLT_INT, SLT_VOID},
	{"GETSTATUS", SLT_INT, SCI_GETSTATUS, SLT_VOID, SLT_VOID},
	{"SETMOUSEDOWNCAPTURES", SLT_VOID, SCI_SETMOUSEDOWNCAPTURES, SLT_BOOL, SLT_VOID},
	{"GETMOUSEDOWNCAPTURES", SLT_BOOL, SCI_GETMOUSEDOWNCAPTURES, SLT_VOID, SLT_VOID},
	{"SETMOUSEWHEELCAPTURES", SLT_VOID, SCI_SETMOUSEWHEELCAPTURES, SLT_BOOL, SLT_VOID},
	{"GETMOUSEWHEELCAPTURES", SLT_BOOL, SCI_GETMOUSEWHEELCAPTURES, SLT_VOID, SLT_VOID},
	{"SETCURSOR", SLT_VOID, SCI_SETCURSOR, SLT_INT, SLT_VOID},
	{"GETCURSOR", SLT_INT, SCI_GETCURSOR, SLT_VOID, SLT_VOID},
	{"SETCONTROLCHARSYMBOL", SLT_VOID, SCI_SETCONTROLCHARSYMBOL, SLT_INT, SLT_VOID},
	{"GETCONTROLCHARSYMBOL", SLT_INT, SCI_GETCONTROLCHARSYMBOL, SLT_VOID, SLT_VOID},
	{"WORDPARTLEFT", SLT_VOID, SCI_WORDPARTLEFT, SLT_VOID, SLT_VOID},
	{"WORDPARTLEFTEXTEND", SLT_VOID, SCI_WORDPARTLEFTEXTEND, SLT_VOID, SLT_VOID},
	{"WORDPARTRIGHT", SLT_VOID, SCI_WORDPARTRIGHT, SLT_VOID, SLT_VOID},
	{"WORDPARTRIGHTEXTEND", SLT_VOID, SCI_WORDPARTRIGHTEXTEND, SLT_VOID, SLT_VOID},
	{"SETVISIBLEPOLICY", SLT_VOID, SCI_SETVISIBLEPOLICY, SLT_INT, SLT_INT},
	{"DELLINELEFT", SLT_VOID, SCI_DELLINELEFT, SLT_VOID, SLT_VOID},
	{"DELLINERIGHT", SLT_VOID, SCI_DELLINERIGHT, SLT_VOID, SLT_VOID},
	{"SETXOFFSET", SLT_VOID, SCI_SETXOFFSET, SLT_INT, SLT_VOID},
	{"GETXOFFSET", SLT_INT, SCI_GETXOFFSET, SLT_VOID, SLT_VOID},
	{"CHOOSECARETX", SLT_VOID, SCI_CHOOSECARETX, SLT_VOID, SLT_VOID},
	{"GRABFOCUS", SLT_VOID, SCI_GRABFOCUS, SLT_VOID, SLT_VOID},
	{"SETXCARETPOLICY", SLT_VOID, SCI_SETXCARETPOLICY, SLT_INT, SLT_INT},
	{"SETYCARETPOLICY", SLT_VOID, SCI_SETYCARETPOLICY, SLT_INT, SLT_INT},
	{"SETPRINTWRAPMODE", SLT_VOID, SCI_SETPRINTWRAPMODE, SLT_INT, SLT_VOID},
	{"GETPRINTWRAPMODE", SLT_INT, SCI_GETPRINTWRAPMODE, SLT_VOID, SLT_VOID},
	{"SETHOTSPOTACTIVEFORE", SLT_VOID, SCI_SETHOTSPOTACTIVEFORE, SLT_BOOL, SLT_INT},
	{"GETHOTSPOTACTIVEFORE", SLT_INT, SCI_GETHOTSPOTACTIVEFORE, SLT_VOID, SLT_VOID},
	{"SETHOTSPOTACTIVEBACK", SLT_VOID, SCI_SETHOTSPOTACTIVEBACK, SLT_BOOL, SLT_INT},
	{"GETHOTSPOTACTIVEBACK", SLT_INT, SCI_GETHOTSPOTACTIVEBACK, SLT_VOID, SLT_VOID},
	{"SETHOTSPOTACTIVEUNDERLINE", SLT_VOID, SCI_SETHOTSPOTACTIVEUNDERLINE, SLT_BOOL, SLT_VOID},
	{"GETHOTSPOTACTIVEUNDERLINE", SLT_BOOL, SCI_GETHOTSPOTACTIVEUNDERLINE, SLT_VOID, SLT_VOID},
	{"SETHOTSPOTSINGLELINE", SLT_VOID, SCI_SETHOTSPOTSINGLELINE, SLT_BOOL, SLT_VOID},
	{"GETHOTSPOTSINGLELINE", SLT_BOOL, SCI_GETHOTSPOTSINGLELINE, SLT_VOID, SLT_VOID},
	{"PARADOWN", SLT_VOID, SCI_PARADOWN, SLT_VOID, SLT_VOID},
	{"PARADOWNEXTEND", SLT_VOID, SCI_PARADOWNEXTEND, SLT_VOID, SLT_VOID},
	{"PARAUP", SLT_VOID, SCI_PARAUP, SLT_VOID, SLT_VOID},
	{"PARAUPEXTEND", SLT_VOID, SCI_PARAUPEXTEND, SLT_VOID, SLT_VOID},
	{"POSITIONBEFORE", SLT_INT, SCI_POSITIONBEFORE, SLT_INT, SLT_VOID},
	{"POSITIONAFTER", SLT_INT, SCI_POSITIONAFTER, SLT_INT, SLT_VOID},
	{"POSITIONRELATIVE", SLT_INT, SCI_POSITIONRELATIVE, SLT_INT, SLT_INT},
	{"POSITIONRELATIVECODEUNITS", SLT_INT, SCI_POSITIONRELATIVECODEUNITS, SLT_INT, SLT_INT},
	{"COPYRANGE", SLT_VOID, SCI_COPYRANGE, SLT_INT, SLT_INT},
	{"COPYTEXT", SLT_VOID, SCI_COPYTEXT, SLT_INT, SLT_STRING},
	{"SETSELECTIONMODE", SLT_VOID, SCI_SETSELECTIONMODE, SLT_INT, SLT_VOID},
	{"GETSELECTIONMODE", SLT_INT, SCI_GETSELECTIONMODE, SLT_VOID, SLT_VOID},
	{"GETMOVEEXTENDSSELECTION", SLT_BOOL, SCI_GETMOVEEXTENDSSELECTION, SLT_VOID, SLT_VOID},
	{"GETLINESELSTARTPOSITION", SLT_INT, SCI_GETLINESELSTARTPOSITION, SLT_INT, SLT_VOID},
	{"GETLINESELENDPOSITION", SLT_INT, SCI_GETLINESELENDPOSITION, SLT_INT, SLT_VOID},
	{"LINEDOWNRECTEXTEND", SLT_VOID, SCI_LINEDOWNRECTEXTEND, SLT_VOID, SLT_VOID},
	{"LINEUPRECTEXTEND", SLT_VOID, SCI_LINEUPRECTEXTEND, SLT_VOID, SLT_VOID},
	{"CHARLEFTRECTEXTEND", SLT_VOID, SCI_CHARLEFTRECTEXTEND, SLT_VOID, SLT_VOID},
	{"CHARRIGHTRECTEXTEND", SLT_VOID, SCI_CHARRIGHTRECTEXTEND, SLT_VOID, SLT_VOID},
	{"HOMERECTEXTEND", SLT_VOID, SCI_HOMERECTEXTEND, SLT_VOID, SLT_VOID},
	{"VCHOMERECTEXTEND", SLT_VOID, SCI_VCHOMERECTEXTEND, SLT_VOID, SLT_VOID},
	{"LINEENDRECTEXTEND", SLT_VOID, SCI_LINEENDRECTEXTEND, SLT_VOID, SLT_VOID},
	{"PAGEUPRECTEXTEND", SLT_VOID, SCI_PAGEUPRECTEXTEND, SLT_VOID, SLT_VOID},
	{"PAGEDOWNRECTEXTEND", SLT_VOID, SCI_PAGEDOWNRECTEXTEND, SLT_VOID, SLT_VOID},
	{"STUTTEREDPAGEUP", SLT_VOID, SCI_STUTTEREDPAGEUP, SLT_VOID, SLT_VOID},
	{"STUTTEREDPAGEUPEXTEND", SLT_VOID, SCI_STUTTEREDPAGEUPEXTEND, SLT_VOID, SLT_VOID},
	{"STUTTEREDPAGEDOWN", SLT_VOID, SCI_STUTTEREDPAGEDOWN, SLT_VOID, SLT_VOID},
	{"STUTTEREDPAGEDOWNEXTEND", SLT_VOID, SCI_STUTTEREDPAGEDOWNEXTEND, SLT_VOID, SLT_VOID},
	{"WORDLEFTEND", SLT_VOID, SCI_WORDLEFTEND, SLT_VOID, SLT_VOID},
	{"WORDLEFTENDEXTEND", SLT_VOID, SCI_WORDLEFTENDEXTEND, SLT_VOID, SLT_VOID},
	{"WORDRIGHTEND", SLT_VOID, SCI_WORDRIGHTEND, SLT_VOID, SLT_VOID},
	{"WORDRIGHTENDEXTEND", SLT_VOID, SCI_WORDRIGHTENDEXTEND, SLT_VOID, SLT_VOID},
	{"SETWHITESPACECHARS", SLT_VOID, SCI_SETWHITESPACECHARS, SLT_VOID, SLT_STRING},
	{"GETWHITESPACECHARS", SLT_INT, SCI_GETWHITESPACECHARS, SLT_VOID, SLT_STRINGRESULT},
	{"SETPUNCTUATIONCHARS", SLT_VOID, SCI_SETPUNCTUATIONCHARS, SLT_VOID, SLT_STRING},
	{"GETPUNCTUATIONCHARS", SLT_INT, SCI_GETPUNCTUATIONCHARS, SLT_VOID, SLT_STRINGRESULT},
	{"SETCHARSDEFAULT", SLT_VOID, SCI_SETCHARSDEFAULT, SLT_VOID, SLT_VOID},
	{"AUTOCGETCURRENT", SLT_INT, SCI_AUTOCGETCURRENT, SLT_VOID, SLT_VOID},
	{"AUTOCGETCURRENTTEXT", SLT_INT, SCI_AUTOCGETCURRENTTEXT, SLT_VOID, SLT_STRINGRESULT},
	{"AUTOCSETCASEINSENSITIVEBEHAVIOUR", SLT_VOID, SCI_AUTOCSETCASEINSENSITIVEBEHAVIOUR, SLT_INT, SLT_VOID},
	{"AUTOCGETCASEINSENSITIVEBEHAVIOUR", SLT_INT, SCI_AUTOCGETCASEINSENSITIVEBEHAVIOUR, SLT_VOID, SLT_VOID},
	{"AUTOCSETMULTI", SLT_VOID, SCI_AUTOCSETMULTI, SLT_INT, SLT_VOID},
	{"AUTOCGETMULTI", SLT_INT, SCI_AUTOCGETMULTI, SLT_VOID, SLT_VOID},
	{"AUTOCSETORDER", SLT_VOID, SCI_AUTOCSETORDER, SLT_INT, SLT_VOID},
	{"AUTOCGETORDER", SLT_INT, SCI_AUTOCGETORDER, SLT_VOID, SLT_VOID},
	{"ALLOCATE", SLT_VOID, SCI_ALLOCATE, SLT_INT, SLT_VOID},
	{"TARGETASUTF8", SLT_INT, SCI_TARGETASUTF8, SLT_VOID, SLT_STRINGRESULT},
	{"SETLENGTHFORENCODE", SLT_VOID, SCI_SETLENGTHFORENCODE, SLT_INT, SLT_VOID},
	{"ENCODEDFROMUTF8", SLT_INT, SCI_ENCODEDFROMUTF8, SLT_STRING, SLT_STRINGRESULT},
	{"FINDCOLUMN", SLT_INT, SCI_FINDCOLUMN, SLT_INT, SLT_INT},
	{"GETCARETSTICKY", SLT_INT, SCI_GETCARETSTICKY, SLT_VOID, SLT_VOID},
	{"SETCARETSTICKY", SLT_VOID, SCI_SETCARETSTICKY, SLT_INT, SLT_VOID},
	{"TOGGLECARETSTICKY", SLT_VOID, SCI_TOGGLECARETSTICKY, SLT_VOID, SLT_VOID},
	{"SETPASTECONVERTENDINGS", SLT_VOID, SCI_SETPASTECONVERTENDINGS, SLT_BOOL, SLT_VOID},
	{"GETPASTECONVERTENDINGS", SLT_BOOL, SCI_GETPASTECONVERTENDINGS, SLT_VOID, SLT_VOID},
	{"SELECTIONDUPLICATE", SLT_VOID, SCI_SELECTIONDUPLICATE, SLT_VOID, SLT_VOID},
	{"SETCARETLINEBACKALPHA", SLT_VOID, SCI_SETCARETLINEBACKALPHA, SLT_INT, SLT_VOID},
	{"GETCARETLINEBACKALPHA", SLT_INT, SCI_GETCARETLINEBACKALPHA, SLT_VOID, SLT_VOID},
	{"SETCARETSTYLE", SLT_VOID, SCI_SETCARETSTYLE, SLT_INT, SLT_VOID},
	{"GETCARETSTYLE", SLT_INT, SCI_GETCARETSTYLE, SLT_VOID, SLT_VOID},
	{"SETINDICATORCURRENT", SLT_VOID, SCI_SETINDICATORCURRENT, SLT_INT, SLT_VOID},
	{"GETINDICATORCURRENT", SLT_INT, SCI_GETINDICATORCURRENT, SLT_VOID, SLT_VOID},
	{"SETINDICATORVALUE", SLT_VOID, SCI_SETINDICATORVALUE, SLT_INT, SLT_VOID},
	{"GETINDICATORVALUE", SLT_INT, SCI_GETINDICATORVALUE, SLT_VOID, SLT_VOID},
	{"INDICATORFILLRANGE", SLT_VOID, SCI_INDICATORFILLRANGE, SLT_INT, SLT_INT},
	{"INDICATORCLEARRANGE", SLT_VOID, SCI_INDICATORCLEARRANGE, SLT_INT, SLT_INT},
	{"INDICATORALLONFOR", SLT_INT, SCI_INDICATORALLONFOR, SLT_INT, SLT_VOID},
	{"INDICATORVALUEAT", SLT_INT, SCI_INDICATORVALUEAT, SLT_INT, SLT_INT},
	{"INDICATORSTART", SLT_INT, SCI_INDICATORSTART, SLT_INT, SLT_INT},
	{"INDICATOREND", SLT_INT, SCI_INDICATOREND, SLT_INT, SLT_INT},
	{"SETPOSITIONCACHE", SLT_VOID, SCI_SETPOSITIONCACHE, SLT_INT, SLT_VOID},
	{"GETPOSITIONCACHE", SLT_INT, SCI_GETPOSITIONCACHE, SLT_VOID, SLT_VOID},
	{"COPYALLOWLINE", SLT_VOID, SCI_COPYALLOWLINE, SLT_VOID, SLT_VOID},
	{"GETCHARACTERPOINTER", SLT_INT, SCI_GETCHARACTERPOINTER, SLT_VOID, SLT_VOID},
	{"GETRANGEPOINTER", SLT_INT, SCI_GETRANGEPOINTER, SLT_INT, SLT_INT},
	{"GETGAPPOSITION", SLT_INT, SCI_GETGAPPOSITION, SLT_VOID, SLT_VOID},
	{"INDICSETALPHA", SLT_VOID, SCI_INDICSETALPHA, SLT_INT, SLT_INT},
	{"INDICGETALPHA", SLT_INT, SCI_INDICGETALPHA, SLT_INT, SLT_VOID},
	{"INDICSETOUTLINEALPHA", SLT_VOID, SCI_INDICSETOUTLINEALPHA, SLT_INT, SLT_INT},
	{"INDICGETOUTLINEALPHA", SLT_INT, SCI_INDICGETOUTLINEALPHA, SLT_INT, SLT_VOID},
	{"SETEXTRAASCENT", SLT_VOID, SCI_SETEXTRAASCENT, SLT_INT, SLT_VOID},
	{"GETEXTRAASCENT", SLT_INT, SCI_GETEXTRAASCENT, SLT_VOID, SLT_VOID},
	{"SETEXTRADESCENT", SLT_VOID, SCI_SETEXTRADESCENT, SLT_INT, SLT_VOID},
	{"GETEXTRADESCENT", SLT_INT, SCI_GETEXTRADESCENT, SLT_VOID, SLT_VOID},
	{"MARKERSYMBOLDEFINED", SLT_INT, SCI_MARKERSYMBOLDEFINED, SLT_INT, SLT_VOID},
	{"MARGINSETTEXT", SLT_VOID, SCI_MARGINSETTEXT, SLT_INT, SLT_STRING},
	{"MARGINGETTEXT", SLT_INT, SCI_MARGINGETTEXT, SLT_INT, SLT_STRINGRESULT},
	{"MARGINSETSTYLE", SLT_VOID, SCI_MARGINSETSTYLE, SLT_INT, SLT_INT},
	{"MARGINGETSTYLE", SLT_INT, SCI_MARGINGETSTYLE, SLT_INT, SLT_VOID},
	{"MARGINSETSTYLES", SLT_VOID, SCI_MARGINSETSTYLES, SLT_INT, SLT_STRING},
	{"MARGINGETSTYLES", SLT_INT, SCI_MARGINGETSTYLES, SLT_INT, SLT_STRINGRESULT},
	{"MARGINTEXTCLEARALL", SLT_VOID, SCI_MARGINTEXTCLEARALL, SLT_VOID, SLT_VOID},
	{"MARGINSETSTYLEOFFSET", SLT_VOID, SCI_MARGINSETSTYLEOFFSET, SLT_INT, SLT_VOID},
	{"MARGINGETSTYLEOFFSET", SLT_INT, SCI_MARGINGETSTYLEOFFSET, SLT_VOID, SLT_VOID},
	{"SETMARGINOPTIONS", SLT_VOID, SCI_SETMARGINOPTIONS, SLT_INT, SLT_VOID},
	{"GETMARGINOPTIONS", SLT_INT, SCI_GETMARGINOPTIONS, SLT_VOID, SLT_VOID},
	{"ANNOTATIONSETTEXT", SLT_VOID, SCI_ANNOTATIONSETTEXT, SLT_INT, SLT_STRING},
	{"ANNOTATIONGETTEXT", SLT_INT, SCI_ANNOTATIONGETTEXT, SLT_INT, SLT_STRINGRESULT},
	{"ANNOTATIONSETSTYLE", SLT_VOID, SCI_ANNOTATIONSETSTYLE, SLT_INT, SLT_INT},
	{"ANNOTATIONGETSTYLE", SLT_INT, SCI_ANNOTATIONGETSTYLE, SLT_INT, SLT_VOID},
	{"ANNOTATIONSETSTYLES", SLT_VOID, SCI_ANNOTATIONSETSTYLES, SLT_INT, SLT_STRING},
	{"ANNOTATIONGETSTYLES", SLT_INT, SCI_ANNOTATIONGETSTYLES, SLT_INT, SLT_STRINGRESULT},
	{"ANNOTATIONGETLINES", SLT_INT, SCI_ANNOTATIONGETLINES, SLT_INT, SLT_VOID},
	{"ANNOTATIONCLEARALL", SLT_VOID, SCI_ANNOTATIONCLEARALL, SLT_VOID, SLT_VOID},
	{"ANNOTATIONSETVISIBLE", SLT_VOID, SCI_ANNOTATIONSETVISIBLE, SLT_INT, SLT_VOID},
	{"ANNOTATIONGETVISIBLE", SLT_INT, SCI_ANNOTATIONGETVISIBLE, SLT_VOID, SLT_VOID},
	{"ANNOTATIONSETSTYLEOFFSET", SLT_VOID, SCI_ANNOTATIONSETSTYLEOFFSET, SLT_INT, SLT_VOID},
	{"ANNOTATIONGETSTYLEOFFSET", SLT_INT, SCI_ANNOTATIONGETSTYLEOFFSET, SLT_VOID, SLT_VOID},
	{"RELEASEALLEXTENDEDSTYLES", SLT_VOID, SCI_RELEASEALLEXTENDEDSTYLES, SLT_VOID, SLT_VOID},
	{"ALLOCATEEXTENDEDSTYLES", SLT_INT, SCI_ALLOCATEEXTENDEDSTYLES, SLT_INT, SLT_VOID},
	{"ADDUNDOACTION", SLT_VOID, SCI_ADDUNDOACTION, SLT_INT, SLT_INT},
	{"CHARPOSITIONFROMPOINT", SLT_INT, SCI_CHARPOSITIONFROMPOINT, SLT_INT, SLT_INT},
	{"CHARPOSITIONFROMPOINTCLOSE", SLT_INT, SCI_CHARPOSITIONFROMPOINTCLOSE, SLT_INT, SLT_INT},
	{"SETMOUSESELECTIONRECTANGULARSWITCH", SLT_VOID, SCI_SETMOUSESELECTIONRECTANGULARSWITCH, SLT_BOOL, SLT_VOID},
	{"GETMOUSESELECTIONRECTANGULARSWITCH", SLT_BOOL, SCI_GETMOUSESELECTIONRECTANGULARSWITCH, SLT_VOID, SLT_VOID},
	{"SETMULTIPLESELECTION", SLT_VOID, SCI_SETMULTIPLESELECTION, SLT_BOOL, SLT_VOID},
	{"GETMULTIPLESELECTION", SLT_BOOL, SCI_GETMULTIPLESELECTION, SLT_VOID, SLT_VOID},
	{"SETADDITIONALSELECTIONTYPING", SLT_VOID, SCI_SETADDITIONALSELECTIONTYPING, SLT_BOOL, SLT_VOID},
	{"GETADDITIONALSELECTIONTYPING", SLT_BOOL, SCI_GETADDITIONALSELECTIONTYPING, SLT_VOID, SLT_VOID},
	{"SETADDITIONALCARETSBLINK", SLT_VOID, SCI_SETADDITIONALCARETSBLINK, SLT_BOOL, SLT_VOID},
	{"GETADDITIONALCARETSBLINK", SLT_BOOL, SCI_GETADDITIONALCARETSBLINK, SLT_VOID, SLT_VOID},
	{"SETADDITIONALCARETSVISIBLE", SLT_VOID, SCI_SETADDITIONALCARETSVISIBLE, SLT_BOOL, SLT_VOID},
	{"GETADDITIONALCARETSVISIBLE", SLT_BOOL, SCI_GETADDITIONALCARETSVISIBLE, SLT_VOID, SLT_VOID},
	{"GETSELECTIONS", SLT_INT, SCI_GETSELECTIONS, SLT_VOID, SLT_VOID},
	{"GETSELECTIONEMPTY", SLT_BOOL, SCI_GETSELECTIONEMPTY, SLT_VOID, SLT_VOID},
	{"CLEARSELECTIONS", SLT_VOID, SCI_CLEARSELECTIONS, SLT_VOID, SLT_VOID},
	{"SETSELECTION", SLT_VOID, SCI_SETSELECTION, SLT_INT, SLT_INT},
	{"ADDSELECTION", SLT_VOID, SCI_ADDSELECTION, SLT_INT, SLT_INT},
	{"DROPSELECTIONN", SLT_VOID, SCI_DROPSELECTIONN, SLT_INT, SLT_VOID},
	{"SETMAINSELECTION", SLT_VOID, SCI_SETMAINSELECTION, SLT_INT, SLT_VOID},
	{"GETMAINSELECTION", SLT_INT, SCI_GETMAINSELECTION, SLT_VOID, SLT_VOID},
	{"SETSELECTIONNCARET", SLT_VOID, SCI_SETSELECTIONNCARET, SLT_INT, SLT_INT},
	{"GETSELECTIONNCARET", SLT_INT, SCI_GETSELECTIONNCARET, SLT_INT, SLT_VOID},
	{"SETSELECTIONNANCHOR", SLT_VOID, SCI_SETSELECTIONNANCHOR, SLT_INT, SLT_INT},
	{"GETSELECTIONNANCHOR", SLT_INT, SCI_GETSELECTIONNANCHOR, SLT_INT, SLT_VOID},
	{"SETSELECTIONNCARETVIRTUALSPACE", SLT_VOID, SCI_SETSELECTIONNCARETVIRTUALSPACE, SLT_INT, SLT_INT},
	{"GETSELECTIONNCARETVIRTUALSPACE", SLT_INT, SCI_GETSELECTIONNCARETVIRTUALSPACE, SLT_INT, SLT_VOID},
	{"SETSELECTIONNANCHORVIRTUALSPACE", SLT_VOID, SCI_SETSELECTIONNANCHORVIRTUALSPACE, SLT_INT, SLT_INT},
	{"GETSELECTIONNANCHORVIRTUALSPACE", SLT_INT, SCI_GETSELECTIONNANCHORVIRTUALSPACE, SLT_INT, SLT_VOID},
	{"SETSELECTIONNSTART", SLT_VOID, SCI_SETSELECTIONNSTART, SLT_INT, SLT_INT},
	{"GETSELECTIONNSTART", SLT_INT, SCI_GETSELECTIONNSTART, SLT_INT, SLT_VOID},
	{"SETSELECTIONNEND", SLT_VOID, SCI_SETSELECTIONNEND, SLT_INT, SLT_INT},
	{"GETSELECTIONNEND", SLT_INT, SCI_GETSELECTIONNEND, SLT_INT, SLT_VOID},
	{"SETRECTANGULARSELECTIONCARET", SLT_VOID, SCI_SETRECTANGULARSELECTIONCARET, SLT_INT, SLT_VOID},
	{"GETRECTANGULARSELECTIONCARET", SLT_INT, SCI_GETRECTANGULARSELECTIONCARET, SLT_VOID, SLT_VOID},
	{"SETRECTANGULARSELECTIONANCHOR", SLT_VOID, SCI_SETRECTANGULARSELECTIONANCHOR, SLT_INT, SLT_VOID},
	{"GETRECTANGULARSELECTIONANCHOR", SLT_INT, SCI_GETRECTANGULARSELECTIONANCHOR, SLT_VOID, SLT_VOID},
	{"SETRECTANGULARSELECTIONCARETVIRTUALSPACE", SLT_VOID, SCI_SETRECTANGULARSELECTIONCARETVIRTUALSPACE, SLT_INT, SLT_VOID},
	{"GETRECTANGULARSELECTIONCARETVIRTUALSPACE", SLT_INT, SCI_GETRECTANGULARSELECTIONCARETVIRTUALSPACE, SLT_VOID, SLT_VOID},
	{"SETRECTANGULARSELECTIONANCHORVIRTUALSPACE", SLT_VOID, SCI_SETRECTANGULARSELECTIONANCHORVIRTUALSPACE, SLT_INT, SLT_VOID},
	{"GETRECTANGULARSELECTIONANCHORVIRTUALSPACE", SLT_INT, SCI_GETRECTANGULARSELECTIONANCHORVIRTUALSPACE, SLT_VOID, SLT_VOID},
	{"SETVIRTUALSPACEOPTIONS", SLT_VOID, SCI_SETVIRTUALSPACEOPTIONS, SLT_INT, SLT_VOID},
	{"GETVIRTUALSPACEOPTIONS", SLT_INT, SCI_GETVIRTUALSPACEOPTIONS, SLT_VOID, SLT_VOID},
	{"SETRECTANGULARSELECTIONMODIFIER", SLT_VOID, SCI_SETRECTANGULARSELECTIONMODIFIER, SLT_INT, SLT_VOID},
	{"GETRECTANGULARSELECTIONMODIFIER", SLT_INT, SCI_GETRECTANGULARSELECTIONMODIFIER, SLT_VOID, SLT_VOID},
	{"SETADDITIONALSELFORE", SLT_VOID, SCI_SETADDITIONALSELFORE, SLT_INT, SLT_VOID},
	{"SETADDITIONALSELBACK", SLT_VOID, SCI_SETADDITIONALSELBACK, SLT_INT, SLT_VOID},
	{"SETADDITIONALSELALPHA", SLT_VOID, SCI_SETADDITIONALSELALPHA, SLT_INT, SLT_VOID},
	{"GETADDITIONALSELALPHA", SLT_INT, SCI_GETADDITIONALSELALPHA, SLT_VOID, SLT_VOID},
	{"SETADDITIONALCARETFORE", SLT_VOID, SCI_SETADDITIONALCARETFORE, SLT_INT, SLT_VOID},
	{"GETADDITIONALCARETFORE", SLT_INT, SCI_GETADDITIONALCARETFORE, SLT_VOID, SLT_VOID},
	{"ROTATESELECTION", SLT_VOID, SCI_ROTATESELECTION, SLT_VOID, SLT_VOID},
	{"SWAPMAINANCHORCARET", SLT_VOID, SCI_SWAPMAINANCHORCARET, SLT_VOID, SLT_VOID},
	{"MULTIPLESELECTADDNEXT", SLT_VOID, SCI_MULTIPLESELECTADDNEXT, SLT_VOID, SLT_VOID},
	{"MULTIPLESELECTADDEACH", SLT_VOID, SCI_MULTIPLESELECTADDEACH, SLT_VOID, SLT_VOID},
	{"CHANGELEXERSTATE", SLT_INT, SCI_CHANGELEXERSTATE, SLT_INT, SLT_INT},
	{"CONTRACTEDFOLDNEXT", SLT_INT, SCI_CONTRACTEDFOLDNEXT, SLT_INT, SLT_VOID},
	{"VERTICALCENTRECARET", SLT_VOID, SCI_VERTICALCENTRECARET, SLT_VOID, SLT_VOID},
	{"MOVESELECTEDLINESUP", SLT_VOID, SCI_MOVESELECTEDLINESUP, SLT_VOID, SLT_VOID},
	{"MOVESELECTEDLINESDOWN", SLT_VOID, SCI_MOVESELECTEDLINESDOWN, SLT_VOID, SLT_VOID},
	{"SETIDENTIFIER", SLT_VOID, SCI_SETIDENTIFIER, SLT_INT, SLT_VOID},
	{"GETIDENTIFIER", SLT_INT, SCI_GETIDENTIFIER, SLT_VOID, SLT_VOID},
	{"RGBAIMAGESETWIDTH", SLT_VOID, SCI_RGBAIMAGESETWIDTH, SLT_INT, SLT_VOID},
	{"RGBAIMAGESETHEIGHT", SLT_VOID, SCI_RGBAIMAGESETHEIGHT, SLT_INT, SLT_VOID},
	{"RGBAIMAGESETSCALE", SLT_VOID, SCI_RGBAIMAGESETSCALE, SLT_INT, SLT_VOID},
	{"MARKERDEFINERGBAIMAGE", SLT_VOID, SCI_MARKERDEFINERGBAIMAGE, SLT_INT, SLT_STRING},
	{"REGISTERRGBAIMAGE", SLT_VOID, SCI_REGISTERRGBAIMAGE, SLT_INT, SLT_STRING},
	{"SCROLLTOSTART", SLT_VOID, SCI_SCROLLTOSTART, SLT_VOID, SLT_VOID},
	{"SCROLLTOEND", SLT_VOID, SCI_SCROLLTOEND, SLT_VOID, SLT_VOID},
	{"SETTECHNOLOGY", SLT_VOID, SCI_SETTECHNOLOGY, SLT_INT, SLT_VOID},
	{"GETTECHNOLOGY", SLT_INT, SCI_GETTECHNOLOGY, SLT_VOID, SLT_VOID},
	{"CREATELOADER", SLT_INT, SCI_CREATELOADER, SLT_INT, SLT_INT},
	{"FINDINDICATORSHOW", SLT_VOID, SCI_FINDINDICATORSHOW, SLT_INT, SLT_INT},
	{"FINDINDICATORFLASH", SLT_VOID, SCI_FINDINDICATORFLASH, SLT_INT, SLT_INT},
	{"FINDINDICATORHIDE", SLT_VOID, SCI_FINDINDICATORHIDE, SLT_VOID, SLT_VOID},
	{"VCHOMEDISPLAY", SLT_VOID, SCI_VCHOMEDISPLAY, SLT_VOID, SLT_VOID},
	{"VCHOMEDISPLAYEXTEND", SLT_VOID, SCI_VCHOMEDISPLAYEXTEND, SLT_VOID, SLT_VOID},
	{"GETCARETLINEVISIBLEALWAYS", SLT_BOOL, SCI_GETCARETLINEVISIBLEALWAYS, SLT_VOID, SLT_VOID},
	{"SETCARETLINEVISIBLEALWAYS", SLT_VOID, SCI_SETCARETLINEVISIBLEALWAYS, SLT_BOOL, SLT_VOID},
	{"SETLINEENDTYPESALLOWED", SLT_VOID, SCI_SETLINEENDTYPESALLOWED, SLT_INT, SLT_VOID},
	{"GETLINEENDTYPESALLOWED", SLT_INT, SCI_GETLINEENDTYPESALLOWED, SLT_VOID, SLT_VOID},
	{"GETLINEENDTYPESACTIVE", SLT_INT, SCI_GETLINEENDTYPESACTIVE, SLT_VOID, SLT_VOID},
	{"SETREPRESENTATION", SLT_VOID, SCI_SETREPRESENTATION, SLT_STRING, SLT_STRING},
	{"GETREPRESENTATION", SLT_INT, SCI_GETREPRESENTATION, SLT_STRING, SLT_STRINGRESULT},
	{"CLEARREPRESENTATION", SLT_VOID, SCI_CLEARREPRESENTATION, SLT_STRING, SLT_VOID},
	{"STARTRECORD", SLT_VOID, SCI_STARTRECORD, SLT_VOID, SLT_VOID},
	{"STOPRECORD", SLT_VOID, SCI_STOPRECORD, SLT_VOID, SLT_VOID},
	{"SETLEXER", SLT_VOID, SCI_SETLEXER, SLT_INT, SLT_VOID},
	{"GETLEXER", SLT_INT, SCI_GETLEXER, SLT_VOID, SLT_VOID},
	{"COLOURISE", SLT_VOID, SCI_COLOURISE, SLT_INT, SLT_INT},
	{"SETPROPERTY", SLT_VOID, SCI_SETPROPERTY, SLT_STRING, SLT_STRING},
	{"SETKEYWORDS", SLT_VOID, SCI_SETKEYWORDS, SLT_INT, SLT_STRING},
	{"SETLEXERLANGUAGE", SLT_VOID, SCI_SETLEXERLANGUAGE, SLT_VOID, SLT_STRING},
	{"LOADLEXERLIBRARY", SLT_VOID, SCI_LOADLEXERLIBRARY, SLT_VOID, SLT_STRING},
	{"GETPROPERTY", SLT_INT, SCI_GETPROPERTY, SLT_STRING, SLT_STRINGRESULT},
	{"GETPROPERTYEXPANDED", SLT_INT, SCI_GETPROPERTYEXPANDED, SLT_STRING, SLT_STRINGRESULT},
	{"GETPROPERTYINT", SLT_INT, SCI_GETPROPERTYINT, SLT_STRING, SLT_INT},
	{"GETLEXERLANGUAGE", SLT_INT, SCI_GETLEXERLANGUAGE, SLT_VOID, SLT_STRINGRESULT},
	{"PRIVATELEXERCALL", SLT_INT, SCI_PRIVATELEXERCALL, SLT_INT, SLT_INT},
	{"PROPERTYNAMES", SLT_INT, SCI_PROPERTYNAMES, SLT_VOID, SLT_STRINGRESULT},
	{"PROPERTYTYPE", SLT_INT, SCI_PROPERTYTYPE, SLT_STRING, SLT_VOID},
	{"DESCRIBEPROPERTY", SLT_INT, SCI_DESCRIBEPROPERTY, SLT_STRING, SLT_STRINGRESULT},
	{"DESCRIBEKEYWORDSETS", SLT_INT, SCI_DESCRIBEKEYWORDSETS, SLT_VOID, SLT_STRINGRESULT},
	{"GETLINEENDTYPESSUPPORTED", SLT_INT, SCI_GETLINEENDTYPESSUPPORTED, SLT_VOID, SLT_VOID},
	{"ALLOCATESUBSTYLES", SLT_INT, SCI_ALLOCATESUBSTYLES, SLT_INT, SLT_INT},
	{"GETSUBSTYLESSTART", SLT_INT, SCI_GETSUBSTYLESSTART, SLT_INT, SLT_VOID},
	{"GETSUBSTYLESLENGTH", SLT_INT, SCI_GETSUBSTYLESLENGTH, SLT_INT, SLT_VOID},
	{"GETSTYLEFROMSUBSTYLE", SLT_INT, SCI_GETSTYLEFROMSUBSTYLE, SLT_INT, SLT_VOID},
	{"GETPRIMARYSTYLEFROMSTYLE", SLT_INT, SCI_GETPRIMARYSTYLEFROMSTYLE, SLT_INT, SLT_VOID},
	{"FREESUBSTYLES", SLT_VOID, SCI_FREESUBSTYLES, SLT_VOID, SLT_VOID},
	{"SETIDENTIFIERS", SLT_VOID, SCI_SETIDENTIFIERS, SLT_INT, SLT_STRING},
	{"DISTANCETOSECONDARYSTYLES", SLT_INT, SCI_DISTANCETOSECONDARYSTYLES, SLT_VOID, SLT_VOID},
	{"GETSUBSTYLEBASES", SLT_INT, SCI_GETSUBSTYLEBASES, SLT_VOID, SLT_STRINGRESULT},
	{"GETNAMEDSTYLES", SLT_INT, SCI_GETNAMEDSTYLES, SLT_VOID, SLT_VOID},
	{"NAMEOFSTYLE", SLT_INT, SCI_NAMEOFSTYLE, SLT_INT, SLT_STRINGRESULT},
	{"TAGSOFSTYLE", SLT_INT, SCI_TAGSOFSTYLE, SLT_INT, SLT_STRINGRESULT},
	{"DESCRIPTIONOFSTYLE", SLT_INT, SCI_DESCRIPTIONOFSTYLE, SLT_INT, SLT_STRINGRESULT},
	{"GETLINECHARACTERINDEX", SLT_INT, SCI_GETLINECHARACTERINDEX, SLT_VOID, SLT_VOID},
	{"ALLOCATELINECHARACTERINDEX", SLT_VOID, SCI_ALLOCATELINECHARACTERINDEX, SLT_INT, SLT_VOID},
	{"RELEASELINECHARACTERINDEX", SLT_VOID, SCI_RELEASELINECHARACTERINDEX, SLT_INT, SLT_VOID},
	{"LINEFROMINDEXPOSITION", SLT_INT, SCI_LINEFROMINDEXPOSITION, SLT_INT, SLT_INT},
	{"INDEXPOSITIONFROMLINE", SLT_INT, SCI_INDEXPOSITIONFROMLINE, SLT_INT, SLT_INT},
	{NULL, SLT_LAST, 0, SLT_LAST, SLT_LAST}
};
