/**
 *   Copyright (C) 2009  Cedric Tabin
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef PLUGIN_ENTRY_H
#define PLUGIN_ENTRY_H

/*========================================== INCLUDES ==========================================================*/

#include <stdlib.h>
#include <stdio.h>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include <geanyplugin.h>

#include <libxml/parser.h>
#include <libxml/tree.h>
#include "PrettyPrinter.h"
#include "ConfigUI.h"

/*========================================== PROPERTIES ========================================================*/

/*========================================== FUNCTIONS =========================================================*/

#endif
