
/*
 * glspi_ver.c - This file is part of the Lua scripting plugin for the Geany IDE
 * See the file "geanylua.c" for copyright information.
 */

#define PLUGIN_NAME _("Lua Script")

#define PLUGIN_DESC _("Write and run Lua scripts for custom commands.\nThis plugin currently has no maintainer. Would you like to help by contributing to this plugin?")

#define PLUGIN_VER VERSION

#define PLUGIN_AUTHOR "Jeff Pohlmeyer"

#define MY_GEANY_API_VER 239

#define LUA_MODULE_NAME "geany"

#define DEFAULT_BANNER _("Lua Script Plugin")


#ifndef PLUGIN_EXPORT
#define PLUGIN_EXPORT
#endif

