/* defconf.c generated by valac 0.34.9, the Vala compiler
 * generated from defconf.vala, do not modify */

/*
 * defconf.vala - This file is part of the Geany MultiTerm plugin
 *
 * Copyright (c) 2012 Matthew Brush <matt@geany.org>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */
/*
 * This file contains the default configuration which will be written
 * to the user's config dir if it doesn't exist.
 *
 * Try to make all defaults apply from here rather than hardcoded in
 * other source files.
 */

#include <glib.h>
#include <glib-object.h>
#include "multiterm.h"







