
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef __DH_ENUM_TYPES_H__
#define __DH_ENUM_TYPES_H__

#include <glib-object.h>

G_BEGIN_DECLS
/* Enumerations from "dh-error.h" */
#define DH_TYPE_ERROR	(dh_error_get_type())
GType dh_error_get_type	(void) G_GNUC_CONST;
/* Enumerations from "dh-link.h" */
#define DH_TYPE_LINK_TYPE	(dh_link_type_get_type())
GType dh_link_type_get_type	(void) G_GNUC_CONST;
#define DH_TYPE_LINK_FLAGS	(dh_link_flags_get_type())
GType dh_link_flags_get_type	(void) G_GNUC_CONST;
/* Enumerations from "dh-window.h" */
#define DH_TYPE_OPEN_LINK_FLAGS	(dh_open_link_flags_get_type())
GType dh_open_link_flags_get_type	(void) G_GNUC_CONST;
G_END_DECLS

#endif /* __DH_ENUM_TYPES_H__ */

/* Generated data ends here */

